# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A generated list with all valid material symbols names.

DO NOT EDIT MANUALLY! The list in this file is generated by `scripts/update_material_icon_names.py`.

This module gets lazy-loaded only when a material icon name check is required.
"""

# fmt: off
### MATERIAL ICON NAMES START ###
ALL_MATERIAL_ICONS = {"10k", "10mp", "11mp", "123", "12mp", "13mp", "14mp", "15mp", "16mp", "17mp", "18_up_rating", "18mp", "19mp", "1k", "1k_plus", "1x_mobiledata", "1x_mobiledata_badge", "20mp", "21mp", "22mp", "23mp", "24fps_select", "24mp", "2d", "2k", "2k_plus", "2mp", "30fps", "30fps_select", "360", "3d", "3d_rotation", "3g_mobiledata", "3g_mobiledata_badge", "3k", "3k_plus", "3mp", "3p", "4g_mobiledata", "4g_mobiledata_badge", "4g_plus_mobiledata", "4k", "4k_plus", "4mp", "50mp", "5g", "5g_mobiledata_badge", "5k", "5k_plus", "5mp", "60fps", "60fps_select", "6_ft_apart", "6k", "6k_plus", "6mp", "7k", "7k_plus", "7mp", "8k", "8k_plus", "8mp", "9k", "9k_plus", "9mp", "abc", "ac_unit", "access_alarm", "access_alarms", "access_time", "access_time_filled", "accessibility", "accessibility_new", "accessible", "accessible_forward", "accessible_menu", "account_balance", "account_balance_wallet", "account_box", "account_child", "account_child_invert", "account_circle", "account_circle_filled", "account_circle_off", "account_tree", "action_key", "activity_zone", "acupuncture", "acute", "ad", "ad_group", "ad_group_off", "ad_off", "ad_units", "adaptive_audio_mic", "adaptive_audio_mic_off", "adb", "add", "add_2", "add_a_photo", "add_ad", "add_alarm", "add_alert", "add_box", "add_business", "add_call", "add_card", "add_chart", "add_circle", "add_circle_outline", "add_column_left", "add_column_right", "add_comment", "add_diamond", "add_home", "add_home_work", "add_ic_call", "add_link", "add_location", "add_location_alt", "add_moderator", "add_notes", "add_photo_alternate", "add_reaction", "add_road", "add_row_above", "add_row_below", "add_shopping_cart", "add_task", "add_to_drive", "add_to_home_screen", "add_to_photos", "add_to_queue", "add_triangle", "addchart", "adf_scanner", "adjust", "admin_meds", "admin_panel_settings", "ads_click", "agender", "agriculture", "air", "air_freshener", "air_purifier", "air_purifier_gen", "airline_seat_flat", "airline_seat_flat_angled", "airline_seat_individual_suite", "airline_seat_legroom_extra", "airline_seat_legroom_normal", "airline_seat_legroom_reduced", "airline_seat_recline_extra", "airline_seat_recline_normal", "airline_stops", "airlines", "airplane_ticket", "airplanemode_active", "airplanemode_inactive", "airplay", "airport_shuttle", "airware", "airwave", "alarm", "alarm_add", "alarm_off", "alarm_on", "alarm_pause", "alarm_smart_wake", "album", "align_center", "align_end", "align_flex_center", "align_flex_end", "align_flex_start", "align_horizontal_center", "align_horizontal_left", "align_horizontal_right", "align_items_stretch", "align_justify_center", "align_justify_flex_end", "align_justify_flex_start", "align_justify_space_around", "align_justify_space_between", "align_justify_space_even", "align_justify_stretch", "align_self_stretch", "align_space_around", "align_space_between", "align_space_even", "align_start", "align_stretch", "align_vertical_bottom", "align_vertical_center", "align_vertical_top", "all_inbox", "all_inclusive", "all_match", "all_out", "allergies", "allergy", "alt_route", "alternate_email", "altitude", "ambient_screen", "ambulance", "amend", "amp_stories", "analytics", "anchor", "android", "android_cell_4_bar", "android_cell_4_bar_alert", "android_cell_4_bar_off", "android_cell_4_bar_plus", "android_cell_5_bar", "android_cell_5_bar_alert", "android_cell_5_bar_off", "android_cell_5_bar_plus", "android_cell_dual_4_bar", "android_cell_dual_4_bar_alert", "android_cell_dual_4_bar_plus", "android_cell_dual_5_bar", "android_cell_dual_5_bar_alert", "android_cell_dual_5_bar_plus", "android_wifi_3_bar", "android_wifi_3_bar_alert", "android_wifi_3_bar_lock", "android_wifi_3_bar_off", "android_wifi_3_bar_plus", "android_wifi_3_bar_question", "android_wifi_4_bar", "android_wifi_4_bar_alert", "android_wifi_4_bar_lock", "android_wifi_4_bar_off", "android_wifi_4_bar_plus", "android_wifi_4_bar_question", "animated_images", "animation", "announcement", "antigravity", "aod", "aod_tablet", "aod_watch", "apartment", "api", "apk_document", "apk_install", "app_badging", "app_blocking", "app_promo", "app_registration", "app_settings_alt", "app_shortcut", "apparel", "approval", "approval_delegation", "approval_delegation_off", "apps", "apps_outage", "aq", "aq_indoor", "ar_on_you", "ar_stickers", "architecture", "archive", "area_chart", "arming_countdown", "arrow_and_edge", "arrow_back", "arrow_back_2", "arrow_back_ios", "arrow_back_ios_new", "arrow_circle_down", "arrow_circle_left", "arrow_circle_right", "arrow_circle_up", "arrow_cool_down", "arrow_downward", "arrow_downward_alt", "arrow_drop_down", "arrow_drop_down_circle", "arrow_drop_up", "arrow_forward", "arrow_forward_ios", "arrow_insert", "arrow_left", "arrow_left_alt", "arrow_menu_close", "arrow_menu_open", "arrow_or_edge", "arrow_outward", "arrow_range", "arrow_right", "arrow_right_alt", "arrow_selector_tool", "arrow_shape_up", "arrow_shape_up_stack", "arrow_shape_up_stack_2", "arrow_split", "arrow_top_left", "arrow_top_right", "arrow_upload_progress", "arrow_upload_ready", "arrow_upward", "arrow_upward_alt", "arrow_warm_up", "arrows_input", "arrows_left_right_circle", "arrows_more_down", "arrows_more_up", "arrows_output", "arrows_outward", "arrows_up_down_circle", "art_track", "article", "article_person", "article_shortcut", "artist", "aspect_ratio", "assessment", "assignment", "assignment_add", "assignment_globe", "assignment_ind", "assignment_late", "assignment_return", "assignment_returned", "assignment_turned_in", "assist_walker", "assistant", "assistant_device", "assistant_direction", "assistant_navigation", "assistant_on_hub", "assistant_photo", "assured_workload", "asterisk", "astrophotography_auto", "astrophotography_off", "atm", "atr", "attach_email", "attach_file", "attach_file_add", "attach_file_off", "attach_money", "attachment", "attractions", "attribution", "audio_description", "audio_file", "audio_video_receiver", "audiotrack", "auto_activity_zone", "auto_awesome", "auto_awesome_mosaic", "auto_awesome_motion", "auto_delete", "auto_detect_voice", "auto_draw_solid", "auto_fix", "auto_fix_high", "auto_fix_normal", "auto_fix_off", "auto_graph", "auto_label", "auto_meeting_room", "auto_mode", "auto_read_pause", "auto_read_play", "auto_schedule", "auto_stories", "auto_stories_off", "auto_timer", "auto_towing", "auto_transmission", "auto_videocam", "autofps_select", "automation", "autopause", "autopay", "autoplay", "autorenew", "autostop", "av1", "av_timer", "avc", "avg_pace", "avg_time", "avocado_bean", "award_meal", "award_star", "azm", "b_circle", "baby_changing_station", "back_hand", "back_to_tab", "background_dot_large", "background_dot_small", "background_grid_small", "background_replace", "backlight_high", "backlight_high_off", "backlight_low", "backpack", "backspace", "backup", "backup_table", "badge", "badge_critical_battery", "badminton", "bakery_dining", "balance", "balcony", "ballot", "bar_chart", "bar_chart_4_bars", "bar_chart_off", "barcode", "barcode_reader", "barcode_scanner", "barefoot", "batch_prediction", "bath_bedrock", "bath_outdoor", "bath_private", "bath_public_large", "bath_soak", "bathroom", "bathtub", "battery_0_bar", "battery_1_bar", "battery_20", "battery_2_bar", "battery_30", "battery_3_bar", "battery_4_bar", "battery_50", "battery_5_bar", "battery_60", "battery_6_bar", "battery_80", "battery_90", "battery_alert", "battery_android_0", "battery_android_1", "battery_android_2", "battery_android_3", "battery_android_4", "battery_android_5", "battery_android_6", "battery_android_alert", "battery_android_bolt", "battery_android_frame_1", "battery_android_frame_2", "battery_android_frame_3", "battery_android_frame_4", "battery_android_frame_5", "battery_android_frame_6", "battery_android_frame_alert", "battery_android_frame_bolt", "battery_android_frame_full", "battery_android_frame_plus", "battery_android_frame_question", "battery_android_frame_share", "battery_android_frame_shield", "battery_android_full", "battery_android_plus", "battery_android_question", "battery_android_share", "battery_android_shield", "battery_change", "battery_charging_20", "battery_charging_20_2", "battery_charging_30", "battery_charging_30_2", "battery_charging_50", "battery_charging_50_2", "battery_charging_60", "battery_charging_60_2", "battery_charging_80", "battery_charging_80_2", "battery_charging_90", "battery_charging_full", "battery_charging_full_2", "battery_error", "battery_full", "battery_full_alt", "battery_horiz_000", "battery_horiz_050", "battery_horiz_075", "battery_low", "battery_plus", "battery_profile", "battery_saver", "battery_share", "battery_status_good", "battery_std", "battery_unknown", "battery_vert_005", "battery_vert_020", "battery_vert_050", "battery_very_low", "beach_access", "bed", "bedroom_baby", "bedroom_child", "bedroom_parent", "bedtime", "bedtime_off", "beenhere", "beer_meal", "bento", "bia", "bid_landscape", "bid_landscape_disabled", "bigtop_updates", "bike_dock", "bike_lane", "bike_scooter", "biotech", "blanket", "blender", "blind", "blinds", "blinds_2", "blinds_2_closed", "blinds_closed", "block", "blood_pressure", "bloodtype", "bluetooth", "bluetooth_audio", "bluetooth_connected", "bluetooth_disabled", "bluetooth_drive", "bluetooth_searching", "blur_circular", "blur_linear", "blur_medium", "blur_off", "blur_on", "blur_short", "boat_bus", "boat_railway", "body_fat", "body_system", "bolt", "bolt_boost", "bomb", "book", "book_2", "book_3", "book_4", "book_5", "book_6", "book_online", "book_ribbon", "bookmark", "bookmark_add", "bookmark_added", "bookmark_bag", "bookmark_border", "bookmark_check", "bookmark_flag", "bookmark_heart", "bookmark_manager", "bookmark_remove", "bookmark_stacks", "bookmark_star", "bookmarks", "books_movies_and_music", "border_all", "border_bottom", "border_clear", "border_color", "border_horizontal", "border_inner", "border_left", "border_outer", "border_right", "border_style", "border_top", "border_vertical", "borg", "bottom_app_bar", "bottom_drawer", "bottom_navigation", "bottom_panel_close", "bottom_panel_open", "bottom_right_click", "bottom_sheets", "box", "box_add", "box_edit", "boy", "brand_awareness", "brand_family", "branding_watermark", "breakfast_dining", "breaking_news", "breaking_news_alt_1", "breastfeeding", "brick", "briefcase_meal", "brightness_1", "brightness_2", "brightness_3", "brightness_4", "brightness_5", "brightness_6", "brightness_7", "brightness_alert", "brightness_auto", "brightness_empty", "brightness_high", "brightness_low", "brightness_medium", "bring_your_own_ip", "broadcast_on_home", "broadcast_on_personal", "broken_image", "browse", "browse_activity", "browse_gallery", "browser_not_supported", "browser_updated", "brunch_dining", "brush", "bubble", "bubble_chart", "bubbles", "bucket_check", "bug_report", "build", "build_circle", "bungalow", "burst_mode", "bus_alert", "bus_map_pin", "bus_railway", "business", "business_center", "business_chip", "business_messages", "buttons_alt", "cabin", "cable", "cable_car", "cached", "cadence", "cake", "cake_add", "calculate", "calendar_add_on", "calendar_apps_script", "calendar_check", "calendar_clock", "calendar_lock", "calendar_meal", "calendar_meal_2", "calendar_month", "calendar_today", "calendar_view_day", "calendar_view_month", "calendar_view_week", "call", "call_end", "call_end_alt", "call_log", "call_made", "call_merge", "call_missed", "call_missed_outgoing", "call_quality", "call_received", "call_split", "call_to_action", "camera", "camera_alt", "camera_enhance", "camera_front", "camera_indoor", "camera_outdoor", "camera_rear", "camera_roll", "camera_video", "cameraswitch", "campaign", "camping", "cancel", "cancel_presentation", "cancel_schedule_send", "candle", "candlestick_chart", "cannabis", "captive_portal", "capture", "car_crash", "car_defrost_left", "car_defrost_low_left", "car_defrost_low_right", "car_defrost_mid_left", "car_defrost_mid_low_left", "car_defrost_mid_low_right", "car_defrost_mid_right", "car_defrost_right", "car_fan_low_left", "car_fan_low_mid_left", "car_fan_low_right", "car_fan_mid_left", "car_fan_mid_low_right", "car_fan_mid_right", "car_fan_recirculate", "car_fan_recirculate_2", "car_gear", "car_lock", "car_mirror_heat", "car_rental", "car_repair", "car_tag", "card_giftcard", "card_membership", "card_travel", "cardio_load", "cardiology", "cards", "cards_stack", "cards_star", "carpenter", "carry_on_bag", "carry_on_bag_checked", "carry_on_bag_inactive", "carry_on_bag_question", "cases", "casino", "cast", "cast_connected", "cast_for_education", "cast_pause", "cast_warning", "castle", "category", "category_search", "celebration", "cell_merge", "cell_tower", "cell_wifi", "center_focus_strong", "center_focus_weak", "chair", "chair_alt", "chair_counter", "chair_fireplace", "chair_umbrella", "chalet", "change_circle", "change_history", "charger", "charging_station", "chart_data", "chat", "chat_add_on", "chat_apps_script", "chat_bubble", "chat_bubble_off", "chat_bubble_outline", "chat_dashed", "chat_error", "chat_info", "chat_paste_go", "chat_paste_go_2", "check", "check_alert", "check_box", "check_box_outline_blank", "check_circle", "check_circle_filled", "check_circle_outline", "check_circle_unread", "check_in_out", "check_indeterminate_small", "check_small", "checkbook", "checked_bag", "checked_bag_question", "checklist", "checklist_rtl", "checkroom", "cheer", "chef_hat", "chess", "chess_bishop", "chess_bishop_2", "chess_king", "chess_king_2", "chess_knight", "chess_pawn", "chess_pawn_2", "chess_queen", "chess_rook", "chevron_backward", "chevron_forward", "chevron_left", "chevron_line_up", "chevron_right", "child_care", "child_friendly", "child_hat", "chip_extraction", "chips", "chrome_reader_mode", "chromecast_2", "chromecast_device", "chronic", "church", "cinematic_blur", "circle", "circle_circle", "circle_notifications", "circles", "circles_ext", "clarify", "class", "clean_hands", "cleaning", "cleaning_bucket", "cleaning_services", "clear", "clear_all", "clear_day", "clear_night", "climate_mini_split", "clinical_notes", "clock_arrow_down", "clock_arrow_up", "clock_loader_10", "clock_loader_20", "clock_loader_40", "clock_loader_60", "clock_loader_80", "clock_loader_90", "close", "close_fullscreen", "close_small", "closed_caption", "closed_caption_add", "closed_caption_disabled", "closed_caption_off", "cloud", "cloud_alert", "cloud_circle", "cloud_done", "cloud_download", "cloud_lock", "cloud_off", "cloud_queue", "cloud_sync", "cloud_upload", "cloudy", "cloudy_filled", "cloudy_snowing", "co2", "co_present", "code", "code_blocks", "code_off", "code_xml", "coffee", "coffee_maker", "cognition", "cognition_2", "collapse_all", "collapse_content", "collections", "collections_bookmark", "color_lens", "colorize", "colors", "combine_columns", "comedy_mask", "comic_bubble", "comment", "comment_bank", "comments_disabled", "commit", "communication", "communities", "communities_filled", "commute", "compare", "compare_arrows", "compass_calibration", "component_exchange", "compost", "compress", "computer", "computer_arrow_up", "computer_cancel", "computer_sound", "concierge", "conditions", "confirmation_number", "congenital", "connect_without_contact", "connected_tv", "connecting_airports", "construction", "contact_emergency", "contact_mail", "contact_page", "contact_phone", "contact_phone_filled", "contact_support", "contactless", "contactless_off", "contacts", "contacts_product", "content_copy", "content_cut", "content_paste", "content_paste_go", "content_paste_off", "content_paste_search", "contextual_token", "contextual_token_add", "contract", "contract_delete", "contract_edit", "contrast", "contrast_circle", "contrast_rtl_off", "contrast_square", "control_camera", "control_point", "control_point_duplicate", "controller_gen", "conversation", "conversion_path", "conversion_path_off", "convert_to_text", "conveyor_belt", "cookie", "cookie_off", "cooking", "cool_to_dry", "copy_all", "copyright", "coronavirus", "corporate_fare", "cottage", "counter_0", "counter_1", "counter_2", "counter_3", "counter_4", "counter_5", "counter_6", "counter_7", "counter_8", "counter_9", "countertops", "create", "create_new_folder", "credit_card", "credit_card_clock", "credit_card_gear", "credit_card_heart", "credit_card_off", "credit_score", "crib", "crisis_alert", "crop", "crop_16_9", "crop_3_2", "crop_5_4", "crop_7_5", "crop_9_16", "crop_din", "crop_free", "crop_landscape", "crop_original", "crop_portrait", "crop_rotate", "crop_square", "crossword", "crowdsource", "crown", "cruelty_free", "css", "csv", "currency_bitcoin", "currency_exchange", "currency_franc", "currency_lira", "currency_pound", "currency_ruble", "currency_rupee", "currency_rupee_circle", "currency_yen", "currency_yuan", "curtains", "curtains_closed", "custom_typography", "cut", "cycle", "cyclone", "dangerous", "dark_mode", "dashboard", "dashboard_2", "dashboard_2_edit", "dashboard_2_gear", "dashboard_customize", "data_alert", "data_array", "data_check", "data_exploration", "data_info_alert", "data_loss_prevention", "data_object", "data_saver_off", "data_saver_on", "data_table", "data_thresholding", "data_usage", "database", "database_off", "database_search", "database_upload", "dataset", "dataset_linked", "date_range", "deblur", "deceased", "decimal_decrease", "decimal_increase", "deck", "dehaze", "delete", "delete_forever", "delete_history", "delete_outline", "delete_sweep", "delivery_dining", "delivery_truck_bolt", "delivery_truck_speed", "demography", "density_large", "density_medium", "density_small", "dentistry", "departure_board", "deployed_code", "deployed_code_account", "deployed_code_alert", "deployed_code_history", "deployed_code_update", "dermatology", "description", "deselect", "design_services", "desk", "deskphone", "desktop_access_disabled", "desktop_cloud", "desktop_cloud_stack", "desktop_landscape", "desktop_landscape_add", "desktop_mac", "desktop_portrait", "desktop_windows", "destruction", "details", "detection_and_zone", "detection_and_zone_off", "detector", "detector_alarm", "detector_battery", "detector_co", "detector_offline", "detector_smoke", "detector_status", "developer_board", "developer_board_off", "developer_guide", "developer_mode", "developer_mode_tv", "device_band", "device_hub", "device_reset", "device_thermostat", "device_unknown", "devices", "devices_fold", "devices_fold_2", "devices_off", "devices_other", "devices_wearables", "dew_point", "diagnosis", "diagonal_line", "dialer_sip", "dialogs", "dialpad", "diamond", "diamond_shine", "dictionary", "difference", "digital_out_of_home", "digital_wellbeing", "dine_heart", "dine_in", "dine_lamp", "dining", "dinner_dining", "directions", "directions_alt", "directions_alt_off", "directions_bike", "directions_boat", "directions_boat_filled", "directions_bus", "directions_bus_filled", "directions_car", "directions_car_filled", "directions_off", "directions_railway", "directions_railway_2", "directions_railway_filled", "directions_run", "directions_subway", "directions_subway_filled", "directions_transit", "directions_transit_filled", "directions_walk", "directory_sync", "dirty_lens", "disabled_by_default", "disabled_visible", "disc_full", "discover_tune", "dishwasher", "dishwasher_gen", "display_external_input", "display_settings", "distance", "diversity_1", "diversity_2", "diversity_3", "diversity_4", "dns", "do_disturb", "do_disturb_alt", "do_disturb_off", "do_disturb_on", "do_not_disturb", "do_not_disturb_alt", "do_not_disturb_off", "do_not_disturb_on", "do_not_disturb_on_total_silence", "do_not_step", "do_not_touch", "dock", "dock_to_bottom", "dock_to_left", "dock_to_right", "docs", "docs_add_on", "docs_apps_script", "document_scanner", "document_search", "domain", "domain_add", "domain_disabled", "domain_verification", "domain_verification_off", "domino_mask", "done", "done_all", "done_outline", "donut_large", "donut_small", "door_back", "door_front", "door_open", "door_sensor", "door_sliding", "doorbell", "doorbell_3p", "doorbell_chime", "double_arrow", "downhill_skiing", "download", "download_2", "download_done", "download_for_offline", "downloading", "draft", "draft_orders", "drafts", "drag_click", "drag_handle", "drag_indicator", "drag_pan", "draw", "draw_abstract", "draw_collage", "drawing_recognition", "dresser", "drive_eta", "drive_export", "drive_file_move", "drive_file_move_outline", "drive_file_move_rtl", "drive_file_rename_outline", "drive_folder_upload", "drive_fusiontable", "drone", "drone_2", "dropdown", "dropper_eye", "dry", "dry_cleaning", "dual_screen", "duo", "dvr", "dynamic_feed", "dynamic_form", "e911_avatar", "e911_emergency", "e_mobiledata", "e_mobiledata_badge", "ear_sound", "earbud_case", "earbud_left", "earbud_right", "earbuds", "earbuds_2", "earbuds_battery", "early_on", "earthquake", "east", "ecg", "ecg_heart", "eco", "eda", "edgesensor_high", "edgesensor_low", "edit", "edit_arrow_down", "edit_arrow_up", "edit_attributes", "edit_audio", "edit_calendar", "edit_document", "edit_location", "edit_location_alt", "edit_note", "edit_notifications", "edit_off", "edit_road", "edit_square", "editor_choice", "egg", "egg_alt", "eject", "elderly", "elderly_woman", "electric_bike", "electric_bolt", "electric_car", "electric_meter", "electric_moped", "electric_rickshaw", "electric_scooter", "electrical_services", "elevation", "elevator", "email", "emergency", "emergency_heat", "emergency_heat_2", "emergency_home", "emergency_recording", "emergency_share", "emergency_share_off", "emoji_emotions", "emoji_events", "emoji_flags", "emoji_food_beverage", "emoji_language", "emoji_nature", "emoji_objects", "emoji_people", "emoji_symbols", "emoji_transportation", "emoticon", "empty_dashboard", "enable", "encrypted", "encrypted_add", "encrypted_add_circle", "encrypted_minus_circle", "encrypted_off", "endocrinology", "energy", "energy_program_saving", "energy_program_time_used", "energy_savings_leaf", "engineering", "enhanced_encryption", "ent", "enterprise", "enterprise_off", "equal", "equalizer", "eraser_size_1", "eraser_size_2", "eraser_size_3", "eraser_size_4", "eraser_size_5", "error", "error_circle_rounded", "error_med", "error_outline", "escalator", "escalator_warning", "euro", "euro_symbol", "ev_charger", "ev_mobiledata_badge", "ev_shadow", "ev_shadow_add", "ev_shadow_minus", "ev_station", "event", "event_available", "event_busy", "event_list", "event_note", "event_repeat", "event_seat", "event_upcoming", "exclamation", "exercise", "exit_to_app", "expand", "expand_all", "expand_circle_down", "expand_circle_right", "expand_circle_up", "expand_content", "expand_less", "expand_more", "expansion_panels", "expension_panels", "experiment", "explicit", "explore", "explore_nearby", "explore_off", "explosion", "export_notes", "exposure", "exposure_neg_1", "exposure_neg_2", "exposure_plus_1", "exposure_plus_2", "exposure_zero", "extension", "extension_off", "eye_tracking", "eyebrow", "eyeglasses", "eyeglasses_2", "eyeglasses_2_sound", "face", "face_2", "face_3", "face_4", "face_5", "face_6", "face_down", "face_left", "face_nod", "face_retouching_natural", "face_retouching_off", "face_right", "face_shake", "face_unlock", "face_up", "fact_check", "factory", "falling", "familiar_face_and_zone", "family_group", "family_history", "family_home", "family_link", "family_restroom", "family_star", "fan_focus", "fan_indirect", "farsight_digital", "fast_forward", "fast_rewind", "fastfood", "faucet", "favorite", "favorite_border", "fax", "feature_search", "featured_play_list", "featured_seasonal_and_gifts", "featured_video", "feed", "feedback", "female", "femur", "femur_alt", "fence", "fertile", "festival", "fiber_dvr", "fiber_manual_record", "fiber_new", "fiber_pin", "fiber_smart_record", "file_copy", "file_copy_off", "file_download", "file_download_done", "file_download_off", "file_export", "file_json", "file_map", "file_map_stack", "file_open", "file_png", "file_present", "file_save", "file_save_off", "file_upload", "file_upload_off", "files", "filter", "filter_1", "filter_2", "filter_3", "filter_4", "filter_5", "filter_6", "filter_7", "filter_8", "filter_9", "filter_9_plus", "filter_alt", "filter_alt_off", "filter_arrow_right", "filter_b_and_w", "filter_center_focus", "filter_drama", "filter_frames", "filter_hdr", "filter_list", "filter_list_alt", "filter_list_off", "filter_none", "filter_retrolux", "filter_tilt_shift", "filter_vintage", "finance", "finance_chip", "finance_mode", "find_in_page", "find_replace", "fingerprint", "fingerprint_off", "fire_check", "fire_extinguisher", "fire_hydrant", "fire_truck", "fireplace", "first_page", "fit_page", "fit_page_height", "fit_page_width", "fit_screen", "fit_width", "fitness_center", "fitness_tracker", "fitness_trackers", "flag", "flag_2", "flag_check", "flag_circle", "flag_filled", "flaky", "flare", "flash_auto", "flash_off", "flash_on", "flashlight_off", "flashlight_on", "flatware", "flex_direction", "flex_no_wrap", "flex_wrap", "flight", "flight_class", "flight_land", "flight_takeoff", "flights_and_hotels", "flightsmode", "flip", "flip_camera_android", "flip_camera_ios", "flip_to_back", "flip_to_front", "float_landscape_2", "float_portrait_2", "flood", "floor", "floor_lamp", "flourescent", "flowchart", "flowsheet", "fluid", "fluid_balance", "fluid_med", "fluorescent", "flutter", "flutter_dash", "flyover", "fmd_bad", "fmd_good", "foggy", "folded_hands", "folder", "folder_check", "folder_check_2", "folder_code", "folder_copy", "folder_data", "folder_delete", "folder_eye", "folder_info", "folder_limited", "folder_managed", "folder_match", "folder_off", "folder_open", "folder_shared", "folder_special", "folder_supervised", "folder_zip", "follow_the_signs", "font_download", "font_download_off", "food_bank", "foot_bones", "footprint", "for_you", "forest", "fork_chart", "fork_left", "fork_right", "fork_spoon", "forklift", "format_align_center", "format_align_justify", "format_align_left", "format_align_right", "format_bold", "format_clear", "format_color_fill", "format_color_reset", "format_color_text", "format_h1", "format_h2", "format_h3", "format_h4", "format_h5", "format_h6", "format_image_back", "format_image_break_left", "format_image_break_right", "format_image_front", "format_image_inline_left", "format_image_inline_right", "format_image_left", "format_image_right", "format_indent_decrease", "format_indent_increase", "format_ink_highlighter", "format_italic", "format_letter_spacing", "format_letter_spacing_2", "format_letter_spacing_standard", "format_letter_spacing_wide", "format_letter_spacing_wider", "format_line_spacing", "format_list_bulleted", "format_list_bulleted_add", "format_list_numbered", "format_list_numbered_rtl", "format_overline", "format_paint", "format_paint_off", "format_paragraph", "format_quote", "format_quote_off", "format_shapes", "format_size", "format_strikethrough", "format_text_clip", "format_text_overflow", "format_text_wrap", "format_textdirection_l_to_r", "format_textdirection_r_to_l", "format_textdirection_vertical", "format_underlined", "format_underlined_squiggle", "forms_add_on", "forms_apps_script", "fort", "forum", "forward", "forward_10", "forward_30", "forward_5", "forward_circle", "forward_media", "forward_to_inbox", "foundation", "fragrance", "frame_bug", "frame_exclamation", "frame_inspect", "frame_person", "frame_person_mic", "frame_person_off", "frame_reload", "frame_source", "free_breakfast", "free_cancellation", "front_hand", "front_loader", "full_coverage", "full_hd", "full_stacked_bar_chart", "fullscreen", "fullscreen_exit", "fullscreen_portrait", "function", "functions", "funicular", "g_mobiledata", "g_mobiledata_badge", "g_translate", "gallery_thumbnail", "game_bumper_left", "game_bumper_right", "game_button_l", "game_button_l1", "game_button_l2", "game_button_r", "game_button_r1", "game_button_r2", "game_button_zl", "game_button_zr", "game_stick_l3", "game_stick_left", "game_stick_r3", "game_stick_right", "game_trigger_left", "game_trigger_right", "gamepad", "gamepad_circle_down", "gamepad_circle_left", "gamepad_circle_right", "gamepad_circle_up", "gamepad_down", "gamepad_left", "gamepad_right", "gamepad_up", "games", "garage", "garage_check", "garage_door", "garage_door_open", "garage_home", "garage_money", "garden_cart", "gas_meter", "gastroenterology", "gate", "gavel", "general_device", "generating_tokens", "genetics", "genres", "gesture", "gesture_select", "get_app", "gif", "gif_2", "gif_box", "girl", "gite", "glass_cup", "globe", "globe_2_cancel", "globe_2_question", "globe_asia", "globe_book", "globe_location_pin", "globe_uk", "glucose", "glyphs", "go_to_line", "golf_course", "gondola_lift", "google_home_devices", "google_plus_reshare", "google_tv_remote", "google_wifi", "gpp_bad", "gpp_good", "gpp_maybe", "gps_fixed", "gps_not_fixed", "gps_off", "grade", "gradient", "grading", "grain", "graph_1", "graph_2", "graph_3", "graph_4", "graph_5", "graph_6", "graph_7", "graph_8", "graphic_eq", "graphic_eq_off", "grass", "grid_3x3", "grid_3x3_off", "grid_4x4", "grid_goldenratio", "grid_guides", "grid_layout_side", "grid_off", "grid_on", "grid_view", "grocery", "group", "group_add", "group_off", "group_remove", "group_search", "group_work", "grouped_bar_chart", "groups", "groups_2", "groups_3", "guardian", "gynecology", "h_mobiledata", "h_mobiledata_badge", "h_plus_mobiledata", "h_plus_mobiledata_badge", "hail", "hallway", "hanami_dango", "hand_bones", "hand_gesture", "hand_gesture_off", "hand_meal", "hand_package", "handheld_controller", "handshake", "handwriting_recognition", "handyman", "hangout_video", "hangout_video_off", "hard_disk", "hard_drive", "hard_drive_2", "hardware", "hd", "hdr_auto", "hdr_auto_select", "hdr_enhanced_select", "hdr_off", "hdr_off_select", "hdr_on", "hdr_on_select", "hdr_plus", "hdr_plus_off", "hdr_strong", "hdr_weak", "head_mounted_device", "headphones", "headphones_battery", "headset", "headset_mic", "headset_off", "healing", "health_and_beauty", "health_and_safety", "health_cross", "health_metrics", "heap_snapshot_large", "heap_snapshot_multiple", "heap_snapshot_thumbnail", "hearing", "hearing_aid", "hearing_aid_disabled", "hearing_aid_disabled_left", "hearing_aid_left", "hearing_disabled", "heart_broken", "heart_check", "heart_minus", "heart_plus", "heart_smile", "heat", "heat_pump", "heat_pump_balance", "height", "helicopter", "help", "help_center", "help_clinic", "help_outline", "hematology", "hevc", "hexagon", "hide", "hide_image", "hide_source", "high_chair", "high_density", "high_quality", "high_res", "highlight", "highlight_alt", "highlight_keyboard_focus", "highlight_mouse_cursor", "highlight_off", "highlight_text_cursor", "highlighter_size_1", "highlighter_size_2", "highlighter_size_3", "highlighter_size_4", "highlighter_size_5", "hiking", "history", "history_2", "history_edu", "history_off", "history_toggle_off", "hive", "hls", "hls_off", "holiday_village", "home", "home_and_garden", "home_app_logo", "home_filled", "home_health", "home_improvement_and_tools", "home_iot_device", "home_max", "home_max_dots", "home_mini", "home_pin", "home_repair_service", "home_speaker", "home_storage", "home_storage_gear", "home_work", "horizontal_align_center", "horizontal_align_left", "horizontal_align_right", "horizontal_distribute", "horizontal_rule", "horizontal_split", "host", "hot_tub", "hotel", "hotel_class", "hourglass", "hourglass_arrow_down", "hourglass_arrow_up", "hourglass_bottom", "hourglass_check", "hourglass_disabled", "hourglass_empty", "hourglass_full", "hourglass_pause", "hourglass_top", "house", "house_siding", "house_with_shield", "houseboat", "household_supplies", "hov", "how_to_reg", "how_to_vote", "hr_resting", "html", "http", "https", "hub", "humerus", "humerus_alt", "humidity_high", "humidity_indoor", "humidity_low", "humidity_mid", "humidity_percentage", "hvac", "hvac_max_defrost", "ice_skating", "icecream", "id_card", "identity_aware_proxy", "identity_platform", "ifl", "iframe", "iframe_off", "image", "image_arrow_up", "image_aspect_ratio", "image_inset", "image_not_supported", "image_search", "imagesearch_roller", "imagesmode", "immunology", "import_contacts", "import_export", "important_devices", "in_home_mode", "inactive_order", "inbox", "inbox_customize", "inbox_text", "inbox_text_asterisk", "inbox_text_person", "inbox_text_share", "incomplete_circle", "indeterminate_check_box", "indeterminate_question_box", "info", "info_i", "infrared", "ink_eraser", "ink_eraser_off", "ink_highlighter", "ink_highlighter_move", "ink_marker", "ink_pen", "ink_selection", "inpatient", "input", "input_circle", "insert_chart", "insert_chart_filled", "insert_chart_outlined", "insert_comment", "insert_drive_file", "insert_emoticon", "insert_invitation", "insert_link", "insert_page_break", "insert_photo", "insert_text", "insights", "install_desktop", "install_mobile", "instant_mix", "integration_instructions", "interactive_space", "interests", "interpreter_mode", "inventory", "inventory_2", "invert_colors", "invert_colors_off", "ios", "ios_share", "iron", "iso", "jamboard_kiosk", "japanese_curry", "japanese_flag", "javascript", "join", "join_full", "join_inner", "join_left", "join_right", "joystick", "jump_to_element", "kanji_alcohol", "kayaking", "kebab_dining", "keep", "keep_off", "keep_pin", "keep_public", "kettle", "key", "key_off", "key_vertical", "key_visualizer", "keyboard", "keyboard_alt", "keyboard_arrow_down", "keyboard_arrow_left", "keyboard_arrow_right", "keyboard_arrow_up", "keyboard_backspace", "keyboard_capslock", "keyboard_capslock_badge", "keyboard_command_key", "keyboard_control_key", "keyboard_double_arrow_down", "keyboard_double_arrow_left", "keyboard_double_arrow_right", "keyboard_double_arrow_up", "keyboard_external_input", "keyboard_full", "keyboard_hide", "keyboard_keys", "keyboard_lock", "keyboard_lock_off", "keyboard_off", "keyboard_onscreen", "keyboard_option_key", "keyboard_previous_language", "keyboard_return", "keyboard_tab", "keyboard_tab_rtl", "keyboard_voice", "kid_star", "king_bed", "kitchen", "kitesurfing", "lab_panel", "lab_profile", "lab_research", "label", "label_important", "label_important_outline", "label_off", "label_outline", "labs", "lan", "landscape", "landscape_2", "landscape_2_edit", "landscape_2_off", "landslide", "language", "language_chinese_array", "language_chinese_cangjie", "language_chinese_dayi", "language_chinese_pinyin", "language_chinese_quick", "language_chinese_wubi", "language_french", "language_gb_english", "language_international", "language_japanese_kana", "language_korean_latin", "language_pinyin", "language_spanish", "language_us", "language_us_colemak", "language_us_dvorak", "laps", "laptop", "laptop_car", "laptop_chromebook", "laptop_mac", "laptop_windows", "lasso_select", "last_page", "launch", "laundry", "layers", "layers_clear", "lda", "leaderboard", "leak_add", "leak_remove", "left_click", "left_panel_close", "left_panel_open", "legend_toggle", "lens", "lens_blur", "letter_switch", "library_add", "library_add_check", "library_books", "library_music", "license", "lift_to_talk", "light", "light_group", "light_group_2", "light_mode", "light_off", "lightbulb", "lightbulb_2", "lightbulb_circle", "lightbulb_outline", "lightning_stand", "lightstrip", "line_axis", "line_curve", "line_end", "line_end_arrow", "line_end_arrow_notch", "line_end_circle", "line_end_diamond", "line_end_square", "line_start", "line_start_arrow", "line_start_arrow_notch", "line_start_circle", "line_start_diamond", "line_start_square", "line_style", "line_weight", "linear_scale", "link", "link_2", "link_off", "linked_camera", "linked_services", "lips", "liquor", "list", "list_alt", "list_alt_add", "list_alt_check", "list_arrow", "lists", "live_help", "live_tv", "living", "local_activity", "local_airport", "local_atm", "local_bar", "local_cafe", "local_car_wash", "local_convenience_store", "local_dining", "local_drink", "local_fire_department", "local_florist", "local_gas_station", "local_grocery_store", "local_hospital", "local_hotel", "local_laundry_service", "local_library", "local_mall", "local_movies", "local_offer", "local_parking", "local_pharmacy", "local_phone", "local_pizza", "local_play", "local_police", "local_post_office", "local_printshop", "local_see", "local_shipping", "local_taxi", "location_automation", "location_away", "location_chip", "location_city", "location_disabled", "location_home", "location_off", "location_on", "location_pin", "location_searching", "locator_tag", "lock", "lock_clock", "lock_open", "lock_open_circle", "lock_open_right", "lock_outline", "lock_person", "lock_reset", "login", "logo_dev", "logout", "looks", "looks_3", "looks_4", "looks_5", "looks_6", "looks_one", "looks_two", "loop", "loupe", "low_density", "low_priority", "lowercase", "loyalty", "lte_mobiledata", "lte_mobiledata_badge", "lte_plus_mobiledata", "lte_plus_mobiledata_badge", "luggage", "lunch_dining", "lyrics", "macro_auto", "macro_off", "magic_button", "magic_exchange", "magic_tether", "magnification_large", "magnification_small", "magnify_docked", "magnify_fullscreen", "mail", "mail_asterisk", "mail_lock", "mail_off", "mail_outline", "mail_shield", "male", "man", "man_2", "man_3", "man_4", "manage_accounts", "manage_history", "manage_search", "manga", "manufacturing", "map", "map_pin_heart", "map_pin_review", "map_search", "maps_home_work", "maps_ugc", "margin", "mark_as_unread", "mark_chat_read", "mark_chat_unread", "mark_email_read", "mark_email_unread", "mark_unread_chat_alt", "markdown", "markdown_copy", "markdown_paste", "markunread", "markunread_mailbox", "masked_transitions", "masked_transitions_add", "masks", "massage", "match_case", "match_case_off", "match_word", "matter", "maximize", "meal_dinner", "meal_lunch", "measuring_tape", "media_bluetooth_off", "media_bluetooth_on", "media_link", "media_output", "media_output_off", "mediation", "medical_information", "medical_mask", "medical_services", "medication", "medication_liquid", "meeting_room", "memory", "memory_alt", "menstrual_health", "menu", "menu_book", "menu_book_2", "menu_open", "merge", "merge_type", "message", "metabolism", "metro", "mfg_nest_yale_lock", "mic", "mic_alert", "mic_double", "mic_external_off", "mic_external_on", "mic_gear", "mic_none", "mic_off", "microbiology", "microwave", "microwave_gen", "military_tech", "mimo", "mimo_disconnect", "mindfulness", "minimize", "minor_crash", "mintmark", "missed_video_call", "missed_video_call_filled", "missing_controller", "mist", "mitre", "mixture_med", "mms", "mobile", "mobile_2", "mobile_3", "mobile_alert", "mobile_arrow_down", "mobile_arrow_right", "mobile_arrow_up_right", "mobile_block", "mobile_camera", "mobile_camera_front", "mobile_camera_rear", "mobile_cancel", "mobile_cast", "mobile_charge", "mobile_chat", "mobile_check", "mobile_code", "mobile_dots", "mobile_friendly", "mobile_gear", "mobile_hand", "mobile_hand_left", "mobile_hand_left_off", "mobile_hand_off", "mobile_info", "mobile_landscape", "mobile_layout", "mobile_lock_landscape", "mobile_lock_portrait", "mobile_loupe", "mobile_menu", "mobile_off", "mobile_question", "mobile_rotate", "mobile_rotate_lock", "mobile_screen_share", "mobile_screensaver", "mobile_sensor_hi", "mobile_sensor_lo", "mobile_share", "mobile_share_stack", "mobile_sound", "mobile_sound_2", "mobile_sound_off", "mobile_speaker", "mobile_text", "mobile_text_2", "mobile_theft", "mobile_ticket", "mobile_unlock", "mobile_vibrate", "mobile_wrench", "mobiledata_arrows", "mobiledata_off", "mode", "mode_comment", "mode_cool", "mode_cool_off", "mode_dual", "mode_edit", "mode_edit_outline", "mode_fan", "mode_fan_2", "mode_fan_off", "mode_heat", "mode_heat_cool", "mode_heat_off", "mode_night", "mode_of_travel", "mode_off_on", "mode_standby", "model_training", "modeling", "monetization_on", "money", "money_bag", "money_off", "money_off_csred", "money_range", "monitor", "monitor_heart", "monitor_weight", "monitor_weight_gain", "monitor_weight_loss", "monitoring", "monochrome_photos", "monorail", "mood", "mood_bad", "mood_heart", "moon_stars", "mop", "moped", "moped_package", "more", "more_down", "more_horiz", "more_time", "more_up", "more_vert", "mosque", "motion_blur", "motion_mode", "motion_photos_auto", "motion_photos_off", "motion_photos_on", "motion_photos_pause", "motion_photos_paused", "motion_play", "motion_sensor_active", "motion_sensor_alert", "motion_sensor_idle", "motion_sensor_urgent", "motorcycle", "mountain_flag", "mountain_steam", "mouse", "mouse_lock", "mouse_lock_off", "move", "move_down", "move_group", "move_item", "move_location", "move_selection_down", "move_selection_left", "move_selection_right", "move_selection_up", "move_to_inbox", "move_up", "moved_location", "movie", "movie_creation", "movie_edit", "movie_edit_off", "movie_filter", "movie_info", "movie_off", "movie_speaker", "moving", "moving_beds", "moving_ministry", "mp", "multicooker", "multiline_chart", "multimodal_hand_eye", "multiple_airports", "multiple_stop", "museum", "music_cast", "music_history", "music_note", "music_note_2", "music_note_add", "music_off", "music_video", "my_location", "mystery", "nat", "nature", "nature_people", "navigate_before", "navigate_next", "navigation", "near_me", "near_me_disabled", "nearby", "nearby_error", "nearby_off", "nephrology", "nest_audio", "nest_cam_floodlight", "nest_cam_indoor", "nest_cam_iq", "nest_cam_iq_outdoor", "nest_cam_magnet_mount", "nest_cam_outdoor", "nest_cam_stand", "nest_cam_wall_mount", "nest_cam_wired_stand", "nest_clock_farsight_analog", "nest_clock_farsight_digital", "nest_connect", "nest_detect", "nest_display", "nest_display_max", "nest_doorbell_visitor", "nest_eco_leaf", "nest_farsight_cool", "nest_farsight_dual", "nest_farsight_eco", "nest_farsight_heat", "nest_farsight_seasonal", "nest_farsight_weather", "nest_found_savings", "nest_gale_wifi", "nest_heat_link_e", "nest_heat_link_gen_3", "nest_hello_doorbell", "nest_locator_tag", "nest_mini", "nest_multi_room", "nest_protect", "nest_remote", "nest_remote_comfort_sensor", "nest_secure_alarm", "nest_sunblock", "nest_tag", "nest_thermostat", "nest_thermostat_e_eu", "nest_thermostat_gen_3", "nest_thermostat_sensor", "nest_thermostat_sensor_eu", "nest_thermostat_zirconium_eu", "nest_true_radiant", "nest_wake_on_approach", "nest_wake_on_press", "nest_wifi_gale", "nest_wifi_mistral", "nest_wifi_point", "nest_wifi_point_vento", "nest_wifi_pro", "nest_wifi_pro_2", "nest_wifi_router", "network_cell", "network_check", "network_intel_node", "network_intelligence", "network_intelligence_history", "network_intelligence_update", "network_locked", "network_manage", "network_node", "network_ping", "network_wifi", "network_wifi_1_bar", "network_wifi_1_bar_locked", "network_wifi_2_bar", "network_wifi_2_bar_locked", "network_wifi_3_bar", "network_wifi_3_bar_locked", "network_wifi_locked", "neurology", "new_label", "new_releases", "new_window", "news", "newsmode", "newspaper", "newsstand", "next_plan", "next_week", "nfc", "nfc_off", "night_shelter", "night_sight_auto", "night_sight_auto_off", "night_sight_max", "nightlife", "nightlight", "nightlight_round", "nights_stay", "no_accounts", "no_adult_content", "no_backpack", "no_crash", "no_drinks", "no_encryption", "no_encryption_gmailerrorred", "no_flash", "no_food", "no_luggage", "no_meals", "no_meeting_room", "no_photography", "no_sim", "no_sound", "no_stroller", "no_transfer", "noise_aware", "noise_control_off", "noise_control_on", "nordic_walking", "north", "north_east", "north_west", "not_accessible", "not_accessible_forward", "not_interested", "not_listed_location", "not_started", "note", "note_add", "note_alt", "note_stack", "note_stack_add", "notes", "notification_add", "notification_audio", "notification_audio_off", "notification_important", "notification_multiple", "notification_settings", "notification_sound", "notifications", "notifications_active", "notifications_none", "notifications_off", "notifications_paused", "notifications_unread", "numbers", "nutrition", "ods", "odt", "offline_bolt", "offline_pin", "offline_pin_off", "offline_share", "oil_barrel", "okonomiyaki", "on_device_training", "on_hub_device", "oncology", "ondemand_video", "online_prediction", "onsen", "opacity", "open_in_browser", "open_in_full", "open_in_new", "open_in_new_down", "open_in_new_off", "open_in_phone", "open_jam", "open_run", "open_with", "ophthalmology", "oral_disease", "orbit", "order_approve", "order_play", "orders", "orthopedics", "other_admission", "other_houses", "outbound", "outbox", "outbox_alt", "outdoor_garden", "outdoor_grill", "outgoing_mail", "outlet", "outlined_flag", "outpatient", "outpatient_med", "output", "output_circle", "oven", "oven_gen", "overview", "overview_key", "owl", "oxygen_saturation", "p2p", "pace", "pacemaker", "package", "package_2", "padding", "padel", "page_control", "page_footer", "page_header", "page_info", "page_menu_ios", "pageless", "pages", "pageview", "paid", "palette", "pallet", "pan_tool", "pan_tool_alt", "pan_zoom", "panorama", "panorama_fish_eye", "panorama_horizontal", "panorama_photosphere", "panorama_vertical", "panorama_wide_angle", "paragliding", "parent_child_dining", "park", "parking_meter", "parking_sign", "parking_valet", "partly_cloudy_day", "partly_cloudy_night", "partner_exchange", "partner_heart", "partner_reports", "party_mode", "passkey", "passport", "password", "password_2", "password_2_off", "patient_list", "pattern", "pause", "pause_circle", "pause_circle_filled", "pause_circle_outline", "pause_presentation", "payment", "payment_arrow_down", "payment_card", "payments", "pedal_bike", "pediatrics", "pen_size_1", "pen_size_2", "pen_size_3", "pen_size_4", "pen_size_5", "pending", "pending_actions", "pentagon", "people", "people_alt", "people_outline", "percent", "percent_discount", "performance_max", "pergola", "perm_camera_mic", "perm_contact_calendar", "perm_data_setting", "perm_device_information", "perm_identity", "perm_media", "perm_phone_msg", "perm_scan_wifi", "person", "person_2", "person_3", "person_4", "person_add", "person_add_alt", "person_add_disabled", "person_alert", "person_apron", "person_book", "person_cancel", "person_celebrate", "person_check", "person_edit", "person_filled", "person_heart", "person_off", "person_outline", "person_pin", "person_pin_circle", "person_play", "person_raised_hand", "person_remove", "person_search", "person_shield", "person_text", "personal_bag", "personal_bag_off", "personal_bag_question", "personal_injury", "personal_places", "personal_video", "pest_control", "pest_control_rodent", "pet_supplies", "pets", "phishing", "phone", "phone_alt", "phone_android", "phone_bluetooth_speaker", "phone_callback", "phone_cancel", "phone_disabled", "phone_enabled", "phone_forwarded", "phone_in_talk", "phone_iphone", "phone_locked", "phone_missed", "phone_paused", "phonelink", "phonelink_erase", "phonelink_lock", "phonelink_off", "phonelink_ring", "phonelink_ring_off", "phonelink_setup", "photo", "photo_album", "photo_auto_merge", "photo_camera", "photo_camera_back", "photo_camera_front", "photo_filter", "photo_frame", "photo_library", "photo_prints", "photo_size_select_actual", "photo_size_select_large", "photo_size_select_small", "php", "physical_therapy", "piano", "piano_off", "pickleball", "picture_as_pdf", "picture_in_picture", "picture_in_picture_alt", "picture_in_picture_center", "picture_in_picture_large", "picture_in_picture_medium", "picture_in_picture_mobile", "picture_in_picture_off", "picture_in_picture_small", "pie_chart", "pie_chart_filled", "pie_chart_outline", "pie_chart_outlined", "pill", "pill_off", "pin", "pin_drop", "pin_end", "pin_invoke", "pinboard", "pinboard_unread", "pinch", "pinch_zoom_in", "pinch_zoom_out", "pip", "pip_exit", "pivot_table_chart", "place", "place_item", "plagiarism", "plane_contrails", "planet", "planner_banner_ad_pt", "planner_review", "play_arrow", "play_circle", "play_disabled", "play_for_work", "play_lesson", "play_music", "play_pause", "play_shapes", "playground", "playground_2", "playing_cards", "playlist_add", "playlist_add_check", "playlist_add_check_circle", "playlist_add_circle", "playlist_play", "playlist_remove", "plug_connect", "plumbing", "plus_one", "podcasts", "podiatry", "podium", "point_of_sale", "point_scan", "poker_chip", "policy", "policy_alert", "poll", "polyline", "polymer", "pool", "portable_wifi_off", "portrait", "position_bottom_left", "position_bottom_right", "position_top_right", "post", "post_add", "potted_plant", "power", "power_input", "power_off", "power_rounded", "power_settings_circle", "power_settings_new", "prayer_times", "precision_manufacturing", "pregnancy", "pregnant_woman", "preliminary", "prescriptions", "present_to_all", "preview", "preview_off", "price_change", "price_check", "print", "print_add", "print_connect", "print_disabled", "print_error", "print_lock", "priority", "priority_high", "privacy", "privacy_tip", "private_connectivity", "problem", "procedure", "process_chart", "production_quantity_limits", "productivity", "progress_activity", "prompt_suggestion", "propane", "propane_tank", "psychiatry", "psychology", "psychology_alt", "public", "public_off", "publish", "published_with_changes", "pulmonology", "pulse_alert", "punch_clock", "push_pin", "qr_code", "qr_code_2", "qr_code_2_add", "qr_code_scanner", "query_builder", "query_stats", "question_answer", "question_exchange", "question_mark", "queue", "queue_music", "queue_play_next", "quick_phrases", "quick_reference", "quick_reference_all", "quick_reorder", "quickreply", "quiet_time", "quiet_time_active", "quiz", "r_mobiledata", "radar", "radio", "radio_button_checked", "radio_button_partial", "radio_button_unchecked", "radiology", "railway_alert", "railway_alert_2", "rainy", "rainy_heavy", "rainy_light", "rainy_snow", "ramen_dining", "ramp_left", "ramp_right", "range_hood", "rate_review", "rate_review_rtl", "raven", "raw_off", "raw_on", "read_more", "readiness_score", "real_estate_agent", "rear_camera", "rebase", "rebase_edit", "receipt", "receipt_long", "receipt_long_off", "recent_actors", "recent_patient", "recenter", "recommend", "record_voice_over", "rectangle", "rectangle_add", "recycling", "redeem", "redo", "reduce_capacity", "refresh", "regular_expression", "relax", "release_alert", "remember_me", "reminder", "reminders_alt", "remote_gen", "remove", "remove_circle", "remove_circle_outline", "remove_done", "remove_from_queue", "remove_moderator", "remove_red_eye", "remove_road", "remove_selection", "remove_shopping_cart", "reopen_window", "reorder", "repartition", "repeat", "repeat_on", "repeat_one", "repeat_one_on", "replace_audio", "replace_image", "replace_video", "replay", "replay_10", "replay_30", "replay_5", "replay_circle_filled", "reply", "reply_all", "report", "report_gmailerrorred", "report_off", "report_problem", "request_page", "request_quote", "reset_brightness", "reset_exposure", "reset_focus", "reset_image", "reset_iso", "reset_settings", "reset_shadow", "reset_shutter_speed", "reset_tv", "reset_white_balance", "reset_wrench", "resize", "resize_window", "respiratory_rate", "responsive_layout", "rest_area", "restart_alt", "restaurant", "restaurant_menu", "restore", "restore_from_trash", "restore_page", "resume", "reviews", "rewarded_ads", "rheumatology", "rib_cage", "rice_bowl", "right_click", "right_panel_close", "right_panel_open", "ring_volume", "ring_volume_filled", "ripples", "road", "robot", "robot_2", "rocket", "rocket_launch", "roller_shades", "roller_shades_closed", "roller_skating", "roofing", "room", "room_preferences", "room_service", "rotate_90_degrees_ccw", "rotate_90_degrees_cw", "rotate_auto", "rotate_left", "rotate_right", "roundabout_left", "roundabout_right", "rounded_corner", "route", "router", "router_off", "routine", "rowing", "rss_feed", "rsvp", "rtt", "rubric", "rule", "rule_folder", "rule_settings", "run_circle", "running_with_errors", "rv_hookup", "safety_check", "safety_check_off", "safety_divider", "sailing", "salinity", "sanitizer", "satellite", "satellite_alt", "sauna", "save", "save_alt", "save_as", "save_clock", "saved_search", "savings", "scale", "scan", "scan_delete", "scanner", "scatter_plot", "scene", "schedule", "schedule_send", "schema", "school", "science", "science_off", "scooter", "score", "scoreboard", "screen_lock_landscape", "screen_lock_portrait", "screen_lock_rotation", "screen_record", "screen_rotation", "screen_rotation_alt", "screen_rotation_up", "screen_search_desktop", "screen_share", "screenshot", "screenshot_frame", "screenshot_frame_2", "screenshot_keyboard", "screenshot_monitor", "screenshot_region", "screenshot_tablet", "script", "scrollable_header", "scuba_diving", "sd", "sd_card", "sd_card_alert", "sd_storage", "sdk", "search", "search_activity", "search_check", "search_check_2", "search_gear", "search_hands_free", "search_insights", "search_off", "seat_cool_left", "seat_cool_right", "seat_heat_left", "seat_heat_right", "seat_vent_left", "seat_vent_right", "security", "security_key", "security_update", "security_update_good", "security_update_warning", "segment", "select", "select_all", "select_check_box", "select_to_speak", "select_window", "select_window_2", "select_window_off", "self_care", "self_improvement", "sell", "sell_cloud", "send", "send_and_archive", "send_money", "send_time_extension", "send_to_mobile", "sensor_door", "sensor_occupied", "sensor_window", "sensors", "sensors_krx", "sensors_krx_off", "sensors_off", "sentiment_calm", "sentiment_content", "sentiment_dissatisfied", "sentiment_excited", "sentiment_extremely_dissatisfied", "sentiment_frustrated", "sentiment_neutral", "sentiment_sad", "sentiment_satisfied", "sentiment_satisfied_alt", "sentiment_stressed", "sentiment_very_dissatisfied", "sentiment_very_satisfied", "sentiment_worried", "serif", "server_person", "service_toolbox", "set_meal", "settings", "settings_accessibility", "settings_account_box", "settings_alert", "settings_applications", "settings_b_roll", "settings_backup_restore", "settings_bluetooth", "settings_brightness", "settings_cell", "settings_cinematic_blur", "settings_ethernet", "settings_heart", "settings_input_antenna", "settings_input_component", "settings_input_composite", "settings_input_hdmi", "settings_input_svideo", "settings_motion_mode", "settings_night_sight", "settings_overscan", "settings_panorama", "settings_phone", "settings_photo_camera", "settings_power", "settings_remote", "settings_seating", "settings_slow_motion", "settings_suggest", "settings_system_daydream", "settings_timelapse", "settings_video_camera", "settings_voice", "settop_component", "severe_cold", "shades", "shades_closed", "shadow", "shadow_add", "shadow_minus", "shape_line", "shape_recognition", "shapes", "share", "share_eta", "share_location", "share_off", "share_reviews", "share_windows", "shaved_ice", "sheets_rtl", "shelf_auto_hide", "shelf_position", "shelves", "shield", "shield_lock", "shield_locked", "shield_moon", "shield_person", "shield_question", "shield_toggle", "shield_watch", "shield_with_heart", "shield_with_house", "shift", "shift_lock", "shift_lock_off", "shoe_cleats", "shop", "shop_2", "shop_two", "shopping_bag", "shopping_bag_speed", "shopping_basket", "shopping_cart", "shopping_cart_checkout", "shopping_cart_off", "shoppingmode", "short_stay", "short_text", "shortcut", "show_chart", "shower", "shuffle", "shuffle_on", "shutter_speed", "shutter_speed_add", "shutter_speed_minus", "sick", "side_navigation", "sign_language", "sign_language_2", "signal_cellular_0_bar", "signal_cellular_1_bar", "signal_cellular_2_bar", "signal_cellular_3_bar", "signal_cellular_4_bar", "signal_cellular_add", "signal_cellular_alt", "signal_cellular_alt_1_bar", "signal_cellular_alt_2_bar", "signal_cellular_alt_off", "signal_cellular_connected_no_internet_0_bar", "signal_cellular_connected_no_internet_4_bar", "signal_cellular_no_sim", "signal_cellular_nodata", "signal_cellular_null", "signal_cellular_off", "signal_cellular_pause", "signal_disconnected", "signal_wifi_0_bar", "signal_wifi_4_bar", "signal_wifi_4_bar_lock", "signal_wifi_bad", "signal_wifi_connected_no_internet_4", "signal_wifi_off", "signal_wifi_statusbar_4_bar", "signal_wifi_statusbar_not_connected", "signal_wifi_statusbar_null", "signature", "signpost", "sim_card", "sim_card_alert", "sim_card_download", "simulation", "single_bed", "sip", "siren", "siren_check", "siren_open", "siren_question", "skateboarding", "skeleton", "skillet", "skillet_cooktop", "skip_next", "skip_previous", "skull", "skull_list", "slab_serif", "sledding", "sleep", "sleep_score", "slide_library", "sliders", "slideshow", "slow_motion_video", "smart_button", "smart_card_reader", "smart_card_reader_off", "smart_display", "smart_outlet", "smart_screen", "smart_toy", "smartphone", "smartphone_camera", "smb_share", "smoke_free", "smoking_rooms", "sms", "sms_failed", "snippet_folder", "snooze", "snowboarding", "snowflake", "snowing", "snowing_heavy", "snowmobile", "snowshoeing", "soap", "soba", "social_distance", "social_leaderboard", "solar_power", "solo_dining", "sort", "sort_by_alpha", "sos", "sound_detection_dog_barking", "sound_detection_glass_break", "sound_detection_loud_sound", "sound_sampler", "soundbar", "soup_kitchen", "source", "source_environment", "source_notes", "south", "south_america", "south_east", "south_west", "spa", "space_bar", "space_dashboard", "space_dashboard_2", "spatial_audio", "spatial_audio_off", "spatial_speaker", "spatial_tracking", "speaker", "speaker_2", "speaker_group", "speaker_notes", "speaker_notes_off", "speaker_phone", "special_character", "specific_gravity", "speech_to_text", "speed", "speed_0_25", "speed_0_2x", "speed_0_5", "speed_0_5x", "speed_0_75", "speed_0_7x", "speed_1_2", "speed_1_25", "speed_1_2x", "speed_1_5", "speed_1_5x", "speed_1_75", "speed_1_7x", "speed_2x", "speed_camera", "spellcheck", "split_scene", "split_scene_down", "split_scene_left", "split_scene_right", "split_scene_up", "splitscreen", "splitscreen_add", "splitscreen_bottom", "splitscreen_landscape", "splitscreen_landscape_add", "splitscreen_left", "splitscreen_portrait", "splitscreen_right", "splitscreen_top", "splitscreen_vertical_add", "spo2", "spoke", "sports", "sports_and_outdoors", "sports_bar", "sports_baseball", "sports_basketball", "sports_cricket", "sports_esports", "sports_football", "sports_golf", "sports_gymnastics", "sports_handball", "sports_hockey", "sports_kabaddi", "sports_martial_arts", "sports_mma", "sports_motorsports", "sports_rugby", "sports_score", "sports_soccer", "sports_tennis", "sports_volleyball", "sprinkler", "sprint", "sql", "square", "square_circle", "square_dot", "square_foot", "ssid_chart", "stack", "stack_group", "stack_hexagon", "stack_off", "stack_star", "stacked_bar_chart", "stacked_email", "stacked_inbox", "stacked_line_chart", "stacks", "stadia_controller", "stadium", "stairs", "stairs_2", "star", "star_border", "star_border_purple500", "star_half", "star_outline", "star_purple500", "star_rate", "star_rate_half", "star_shine", "stars", "stars_2", "start", "stat_0", "stat_1", "stat_2", "stat_3", "stat_minus_1", "stat_minus_2", "stat_minus_3", "stay_current_landscape", "stay_current_portrait", "stay_primary_landscape", "stay_primary_portrait", "steering_wheel_heat", "step", "step_into", "step_out", "step_over", "steppers", "steps", "stethoscope", "stethoscope_arrow", "stethoscope_check", "sticker", "sticker_add", "sticky_note", "sticky_note_2", "stock_media", "stockpot", "stop", "stop_circle", "stop_screen_share", "storage", "store", "store_mall_directory", "storefront", "storm", "straight", "straighten", "strategy", "stream", "stream_apps", "streetview", "stress_management", "strikethrough_s", "stroke_full", "stroke_partial", "stroller", "style", "styler", "stylus", "stylus_brush", "stylus_fountain_pen", "stylus_highlighter", "stylus_laser_pointer", "stylus_note", "stylus_pen", "stylus_pencil", "subdirectory_arrow_left", "subdirectory_arrow_right", "subheader", "subject", "subscript", "subscriptions", "subtitles", "subtitles_gear", "subtitles_off", "subway", "subway_walk", "subwoofer", "summarize", "sunny", "sunny_snowing", "superscript", "supervised_user_circle", "supervised_user_circle_off", "supervisor_account", "support", "support_agent", "surfing", "surgical", "surround_sound", "swap_calls", "swap_driving_apps", "swap_driving_apps_wheel", "swap_horiz", "swap_horizontal_circle", "swap_vert", "swap_vertical_circle", "sweep", "swipe", "swipe_down", "swipe_down_alt", "swipe_left", "swipe_left_2", "swipe_left_alt", "swipe_right", "swipe_right_2", "swipe_right_alt", "swipe_up", "swipe_up_alt", "swipe_vertical", "switch", "switch_access", "switch_access_2", "switch_access_3", "switch_access_shortcut", "switch_access_shortcut_add", "switch_account", "switch_camera", "switch_left", "switch_off", "switch_right", "switch_video", "switches", "sword_rose", "swords", "symptoms", "synagogue", "sync", "sync_alt", "sync_arrow_down", "sync_arrow_up", "sync_desktop", "sync_disabled", "sync_lock", "sync_problem", "sync_saved_locally", "sync_saved_locally_off", "syringe", "system_security_update", "system_security_update_good", "system_security_update_warning", "system_update", "system_update_alt", "tab", "tab_close", "tab_close_inactive", "tab_close_right", "tab_duplicate", "tab_group", "tab_inactive", "tab_move", "tab_new_right", "tab_recent", "tab_search", "tab_unselected", "table", "table_bar", "table_chart", "table_chart_view", "table_convert", "table_edit", "table_eye", "table_lamp", "table_large", "table_restaurant", "table_rows", "table_rows_narrow", "table_sign", "table_view", "tablet", "tablet_android", "tablet_camera", "tablet_mac", "tabs", "tactic", "tag", "tag_faces", "takeout_dining", "takeout_dining_2", "tamper_detection_off", "tamper_detection_on", "tap_and_play", "tapas", "target", "task", "task_alt", "tatami_seat", "taunt", "taxi_alert", "team_dashboard", "temp_preferences_custom", "temp_preferences_eco", "temple_buddhist", "temple_hindu", "tenancy", "terminal", "terminal_2", "terrain", "text_ad", "text_ad_off", "text_compare", "text_decrease", "text_fields", "text_fields_alt", "text_format", "text_increase", "text_rotate_up", "text_rotate_vertical", "text_rotation_angledown", "text_rotation_angleup", "text_rotation_down", "text_rotation_none", "text_select_end", "text_select_jump_to_beginning", "text_select_jump_to_end", "text_select_move_back_character", "text_select_move_back_word", "text_select_move_down", "text_select_move_forward_character", "text_select_move_forward_word", "text_select_move_up", "text_select_start", "text_snippet", "text_to_speech", "text_up", "textsms", "texture", "texture_add", "texture_minus", "theater_comedy", "theaters", "thermometer", "thermometer_add", "thermometer_alert", "thermometer_gain", "thermometer_loss", "thermometer_minus", "thermostat", "thermostat_arrow_down", "thermostat_arrow_up", "thermostat_auto", "thermostat_carbon", "things_to_do", "thread_unread", "threat_intelligence", "thumb_down", "thumb_down_alt", "thumb_down_filled", "thumb_down_off", "thumb_down_off_alt", "thumb_up", "thumb_up_alt", "thumb_up_filled", "thumb_up_off", "thumb_up_off_alt", "thumbnail_bar", "thumbs_up_double", "thumbs_up_down", "thunderstorm", "tibia", "tibia_alt", "tile_large", "tile_medium", "tile_small", "time_auto", "time_to_leave", "timelapse", "timeline", "timer", "timer_1", "timer_10", "timer_10_alt_1", "timer_10_select", "timer_2", "timer_3", "timer_3_alt_1", "timer_3_select", "timer_5", "timer_5_shutter", "timer_arrow_down", "timer_arrow_up", "timer_off", "timer_pause", "timer_play", "tips_and_updates", "tire_repair", "title", "titlecase", "toast", "toc", "today", "toggle_off", "toggle_on", "token", "toll", "tonality", "tonality_2", "toolbar", "tools_flat_head", "tools_installation_kit", "tools_ladder", "tools_level", "tools_phillips", "tools_pliers_wire_stripper", "tools_power_drill", "tools_wrench", "tooltip", "tooltip_2", "top_panel_close", "top_panel_open", "topic", "tornado", "total_dissolved_solids", "touch_app", "touch_double", "touch_long", "touch_triple", "touchpad_mouse", "touchpad_mouse_off", "tour", "toys", "toys_and_games", "toys_fan", "track_changes", "trackpad_input", "trackpad_input_2", "trackpad_input_3", "traffic", "traffic_jam", "trail_length", "trail_length_medium", "trail_length_short", "train", "tram", "transcribe", "transfer_within_a_station", "transform", "transgender", "transit_enterexit", "transit_ticket", "transition_chop", "transition_dissolve", "transition_fade", "transition_push", "transition_slide", "translate", "translate_indic", "transportation", "travel", "travel_explore", "travel_luggage_and_bags", "trending_down", "trending_flat", "trending_up", "triangle_circle", "trip", "trip_origin", "trolley", "trolley_cable_car", "trophy", "troubleshoot", "try", "tsunami", "tsv", "tty", "tune", "tungsten", "turn_left", "turn_right", "turn_sharp_left", "turn_sharp_right", "turn_slight_left", "turn_slight_right", "turned_in", "turned_in_not", "tv", "tv_displays", "tv_gen", "tv_guide", "tv_next", "tv_off", "tv_options_edit_channels", "tv_options_input_settings", "tv_remote", "tv_signin", "tv_with_assistant", "two_pager", "two_pager_store", "two_wheeler", "type_specimen", "u_turn_left", "u_turn_right", "udon", "ulna_radius", "ulna_radius_alt", "umbrella", "unarchive", "undereye", "undo", "unfold_less", "unfold_less_double", "unfold_more", "unfold_more_double", "ungroup", "universal_currency", "universal_currency_alt", "universal_local", "unknown_2", "unknown_5", "unknown_7", "unknown_document", "unknown_med", "unlicense", "unpaved_road", "unpin", "unpublished", "unsubscribe", "upcoming", "update", "update_disabled", "upgrade", "upi_pay", "upload", "upload_2", "upload_file", "uppercase", "urology", "usb", "usb_off", "user_attributes", "vaccines", "vacuum", "vacuum_2", "vacuum_2_on", "valve", "vape_free", "vaping_rooms", "variable_add", "variable_insert", "variable_remove", "variables", "ventilator", "verified", "verified_off", "verified_user", "vertical_align_bottom", "vertical_align_center", "vertical_align_top", "vertical_distribute", "vertical_shades", "vertical_shades_closed", "vertical_split", "vibration", "video_call", "video_camera_back", "video_camera_back_add", "video_camera_front", "video_camera_front_off", "video_chat", "video_file", "video_label", "video_library", "video_search", "video_settings", "video_stable", "video_template", "videocam", "videocam_alert", "videocam_off", "videogame_asset", "videogame_asset_off", "view_agenda", "view_apps", "view_array", "view_carousel", "view_column", "view_column_2", "view_comfy", "view_comfy_alt", "view_compact", "view_compact_alt", "view_cozy", "view_day", "view_headline", "view_in_ar", "view_in_ar_new", "view_in_ar_off", "view_kanban", "view_list", "view_module", "view_object_track", "view_quilt", "view_real_size", "view_sidebar", "view_stream", "view_timeline", "view_week", "vignette", "vignette_2", "villa", "visibility", "visibility_lock", "visibility_off", "vital_signs", "vitals", "vo2_max", "voice_chat", "voice_chat_off", "voice_over_off", "voice_selection", "voice_selection_off", "voicemail", "voicemail_2", "volcano", "volume_down", "volume_down_alt", "volume_mute", "volume_off", "volume_up", "volunteer_activism", "voting_chip", "vpn_key", "vpn_key_alert", "vpn_key_off", "vpn_lock", "vpn_lock_2", "vr180_create2d", "vr180_create2d_off", "vrpano", "wall_art", "wall_lamp", "wallet", "wallpaper", "wallpaper_slideshow", "wand_shine", "wand_stars", "ward", "warehouse", "warning", "warning_amber", "warning_off", "wash", "washoku", "watch", "watch_alert", "watch_arrow", "watch_arrow_down", "watch_button_press", "watch_check", "watch_later", "watch_lock", "watch_off", "watch_screentime", "watch_vibration", "watch_wake", "water", "water_bottle", "water_bottle_large", "water_damage", "water_do", "water_drop", "water_drops", "water_ec", "water_full", "water_heater", "water_lock", "water_loss", "water_lux", "water_medium", "water_orp", "water_ph", "water_pump", "water_voc", "waterfall_chart", "waves", "waving_hand", "wb_auto", "wb_cloudy", "wb_incandescent", "wb_iridescent", "wb_shade", "wb_sunny", "wb_twilight", "wc", "weather_hail", "weather_mix", "weather_snowy", "web", "web_asset", "web_asset_off", "web_stories", "web_traffic", "webhook", "weekend", "weight", "west", "whatshot", "wheat", "wheelchair_pickup", "where_to_vote", "widget_medium", "widget_menu", "widget_small", "widget_width", "widgets", "width", "width_full", "width_normal", "width_wide", "wifi", "wifi_1_bar", "wifi_2_bar", "wifi_add", "wifi_calling", "wifi_calling_1", "wifi_calling_2", "wifi_calling_3", "wifi_calling_bar_1", "wifi_calling_bar_2", "wifi_calling_bar_3", "wifi_channel", "wifi_device", "wifi_find", "wifi_home", "wifi_lock", "wifi_notification", "wifi_off", "wifi_password", "wifi_protected_setup", "wifi_proxy", "wifi_tethering", "wifi_tethering_error", "wifi_tethering_off", "wind_power", "window", "window_closed", "window_open", "window_sensor", "windshield_defrost_auto", "windshield_defrost_front", "windshield_defrost_rear", "windshield_heat_front", "wine_bar", "woman", "woman_2", "work", "work_alert", "work_history", "work_off", "work_outline", "work_update", "workflow", "workspace_premium", "workspaces", "workspaces_outline", "wounds_injuries", "wrap_text", "wrist", "wrong_location", "wysiwyg", "x_circle", "y_circle", "yakitori", "yard", "yoshoku", "your_trips", "youtube_activity", "youtube_searched_for", "zone_person_alert", "zone_person_idle", "zone_person_urgent", "zoom_in", "zoom_in_map", "zoom_out", "zoom_out_map"}
### MATERIAL ICON NAMES END ###



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: streamlit/proto/Feedback.proto
# Protobuf Python Version: 7.34.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    7,
    34,
    1,
    '',
    'streamlit/proto/Feedback.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1estreamlit/proto/Feedback.proto\"\xe4\x01\n\x08\x46\x65\x65\x64\x62\x61\x63k\x12\n\n\x02id\x18\x01 \x01(\t\x12$\n\x04type\x18\x02 \x01(\x0e\x32\x16.Feedback.FeedbackType\x12\x14\n\x07\x64\x65\x66\x61ult\x18\x03 \x01(\rH\x00\x88\x01\x01\x12\x10\n\x08\x64isabled\x18\x04 \x01(\x08\x12\x0f\n\x07\x66orm_id\x18\x05 \x01(\t\x12\x12\n\x05value\x18\x06 \x01(\rH\x01\x88\x01\x01\x12\x11\n\tset_value\x18\x07 \x01(\x08\"0\n\x0c\x46\x65\x65\x64\x62\x61\x63kType\x12\n\n\x06THUMBS\x10\x00\x12\t\n\x05\x46\x41\x43\x45S\x10\x01\x12\t\n\x05STARS\x10\x02\x42\n\n\x08_defaultB\x08\n\x06_valueb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'streamlit.proto.Feedback_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_FEEDBACK']._serialized_start=35
  _globals['_FEEDBACK']._serialized_end=263
  _globals['_FEEDBACK_FEEDBACKTYPE']._serialized_start=193
  _globals['_FEEDBACK_FEEDBACKTYPE']._serialized_end=241
# @@protoc_insertion_point(module_scope)



# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

import os
from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Any

from streamlit import util
from streamlit.errors import StreamlitAPIException

if TYPE_CHECKING:
    from streamlit.runtime.state.common import WidgetCallback


class MarshallComponentException(StreamlitAPIException):
    """Class for exceptions generated during custom component marshalling."""


class BaseCustomComponent(ABC):
    """Interface for CustomComponents."""

    def __init__(
        self,
        name: str,
        path: str | None = None,
        url: str | None = None,
        module_name: str | None = None,
    ) -> None:
        if path is None and url is None:
            raise StreamlitAPIException("Either 'path' or 'url' must be set.")

        self._name = name
        self._path = path
        self._url = url
        self._module_name = module_name

    def __repr__(self) -> str:
        return util.repr_(self)

    def __call__(
        self,
        *args: Any,
        default: Any = None,
        key: str | None = None,
        on_change: WidgetCallback | None = None,
        tab_index: int | None = None,
        **kwargs: Any,
    ) -> Any:
        """An alias for create_instance."""
        return self.create_instance(
            *args,
            default=default,
            key=key,
            on_change=on_change,
            tab_index=tab_index,
            **kwargs,
        )

    @property
    def abspath(self) -> str | None:
        if self._path is None:
            return None
        return os.path.abspath(self._path)

    @property
    def module_name(self) -> str | None:
        return self._module_name

    @property
    def name(self) -> str:
        return self._name

    @property
    def path(self) -> str | None:
        return self._path

    @property
    def url(self) -> str | None:
        return self._url

    def __str__(self) -> str:
        return f"'{self.name}': {self.path if self.path is not None else self.url}"

    def __hash__(self) -> int:
        return hash((self.name, self.path, self.url, self.module_name))

    @abstractmethod
    def __eq__(self, other: object) -> bool:
        """Equality operator."""
        return NotImplemented

    @abstractmethod
    def __ne__(self, other: object) -> bool:
        """Inequality operator."""
        return NotImplemented

    @abstractmethod
    def create_instance(
        self,
        *args: Any,
        default: Any = None,
        key: str | None = None,
        on_change: WidgetCallback | None = None,
        tab_index: int | None = None,
        **kwargs: Any,
    ) -> Any:
        """Create a new instance of the component.

        Parameters
        ----------
        *args
            Must be empty; all args must be named. (This parameter exists to
            enforce correct use of the function.)
        default: any or None
            The default return value for the component. This is returned when
            the component's frontend hasn't yet specified a value with
            `setComponentValue`.
        key: str or None
            If not None, this is the user key we use to generate the
            component's "widget ID".
        on_change: WidgetCallback or None
            An optional callback invoked when the widget's value changes. No arguments are passed to it.
        tab_index: int or None
            Specifies the tab order of the iframe containing the component.
            Possible values are:
            - ``None`` (default): Browser default behavior.
            - ``-1``: Removes the iframe from the natural tab order, but it can still be focused programmatically.
            - ``0`` or positive integer: Includes the iframe in the natural tab order.
        **kwargs
            Keyword args to pass to the component.

        Returns
        -------
        any or None
            The component's widget value.

        Raises
        ------
        MarshallComponentException
            Raised when args is not empty or component cannot be marshalled.
        StreamlitAPIException
            Raised when PyArrow is not installed.

        """
        raise NotImplementedError



# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Company analytics dashboard template.

Demonstrates:
- Interactive dataframe with sparkline columns
- Segmented control for ranking (top spenders, gainers, shrinkers)
- Multi-select pills for account type filtering
- Time window filtering
- Growth score calculation
- Dialog popup for company details

This template uses synthetic data. Replace ``generate_company_data()`` with
your actual data source (e.g., Snowflake queries, CRM APIs, etc.).
"""

from datetime import date, timedelta

import numpy as np
import pandas as pd

import streamlit as st

st.set_page_config(
    page_title="Company Analytics",
    page_icon=":material/business:",
    layout="wide",
)


# =============================================================================
# Synthetic Data Generation (Replace with your data source)
# =============================================================================

COMPANY_NAMES = [
    "Acme Corp",
    "TechFlow Inc",
    "DataDriven Co",
    "CloudFirst Ltd",
    "InnovateTech",
    "ScaleUp Systems",
    "PrimeData Inc",
    "FutureStack",
    "ByteWise Corp",
    "StreamLine Co",
    "Quantum Labs",
    "NexGen Solutions",
    "AlphaMetrics",
    "BetaAnalytics",
    "GammaInsights",
    "DeltaData",
    "OmegaTech",
    "SigmaSoft",
    "ThetaCloud",
    "ZetaDigital",
]

ACCOUNT_TYPES = ["Enterprise", "Growth", "Startup", "Trial", "Internal"]
REGIONS = ["North America", "EMEA", "APAC", "LATAM"]
SEGMENTS = ["Technology", "Finance", "Healthcare", "Retail", "Manufacturing"]


@st.cache_data(ttl=3600)
def generate_company_data(days: int = 90) -> pd.DataFrame:
    """Generate synthetic company usage data.

    Replace this function with your actual data source.
    """
    rng = np.random.default_rng(42)

    end_date = date.today() - timedelta(days=1)
    start_date = end_date - timedelta(days=days)
    dates = pd.date_range(start=start_date, end=end_date, freq="D")

    records = []

    for company in COMPANY_NAMES:
        # Assign static attributes
        account_type = rng.choice(ACCOUNT_TYPES, p=[0.3, 0.25, 0.2, 0.15, 0.1])
        region = rng.choice(REGIONS)
        segment = rng.choice(SEGMENTS)

        # Generate usage pattern
        base_usage = rng.integers(100, 10000)
        growth = rng.uniform(-0.005, 0.01)  # Some companies shrink

        for i, dt in enumerate(dates):
            # Base trend
            trend = base_usage * (1 + growth) ** i

            # Weekly seasonality
            if dt.dayofweek >= 5:
                trend *= 0.3

            # Random noise
            daily_credits = max(0, trend * rng.uniform(0.7, 1.3))

            records.append(
                {
                    "company_name": company,
                    "date": dt,
                    "daily_credits": daily_credits,
                    "account_type": account_type,
                    "region": region,
                    "segment": segment,
                }
            )

    return pd.DataFrame(records)


@st.cache_data(ttl=3600)
def load_company_data() -> pd.DataFrame:
    """Load all company data."""
    return generate_company_data(days=90)


def _calc_growth(trend: list[float]) -> float:
    """Return second-half vs first-half delta (rough trend indicator)."""
    if not trend or len(trend) < 2:
        return 0
    mid = len(trend) // 2
    first_half = sum(trend[:mid]) if mid > 0 else 0
    second_half = sum(trend[mid:])
    return second_half - first_half


def _to_list(val: object) -> list[object]:
    """Wrap a scalar in a single-element list for MultiselectColumn display."""
    return [val] if pd.notna(val) else []  # type: ignore[call-overload]  # ty: ignore[no-matching-overload]


def aggregate_companies(
    df: pd.DataFrame,
    days: int | None = None,
    account_types: list[str] | None = None,
    sort_by: str = "total_credits",
) -> pd.DataFrame:
    """Filter and aggregate company data."""
    result = df.copy()

    # Filter by time window
    if days:
        cutoff = pd.Timestamp.now() - pd.Timedelta(days=days)
        result = result[result["date"] >= cutoff]

    # Filter by account type
    if account_types:
        result = result[result["account_type"].isin(account_types)]

    if result.empty:
        return pd.DataFrame()

    # Aggregate to company level
    agg = (
        result.groupby("company_name")
        .agg(
            total_credits=("daily_credits", "sum"),
            active_days=("date", "nunique"),
            account_type=("account_type", "first"),
            region=("region", "first"),
            segment=("segment", "first"),
        )
        .reset_index()
    )

    # Calculate daily average
    agg["daily_avg"] = agg["total_credits"] / agg["active_days"]

    # Build sparkline data (list of daily values).
    # `include_groups=False` silences a pandas 2.2+ FutureWarning about
    # groupby keys being included in the applied function's input frame.
    sparklines = (
        result.groupby("company_name")  # ty: ignore[no-matching-overload]
        .apply(
            lambda x: x.sort_values("date")["daily_credits"].tolist(),
            include_groups=False,  # type: ignore[call-overload]
        )
        .reset_index()
    )
    sparklines.columns = ["company_name", "usage_trend"]
    agg = agg.merge(sparklines, on="company_name")

    agg["growth_score"] = agg["usage_trend"].apply(_calc_growth)

    # Sort
    if sort_by == "growth_asc":
        agg = agg.sort_values("growth_score", ascending=True)
    elif sort_by == "growth_desc":
        agg = agg.sort_values("growth_score", ascending=False)
    else:
        agg = agg.sort_values("total_credits", ascending=False)

    return agg


def render_company_dialog(
    company_name: str, company_row: pd.Series, df: pd.DataFrame
) -> None:
    """Render company details inside a dialog."""
    company_data = df[df["company_name"] == company_name].sort_values("date")

    if company_data.empty:
        st.warning("No data available for this company.")
        return

    # Company info badges - extract from list format back to single value
    account_type = (
        company_row["account_type"][0] if company_row["account_type"] else "Unknown"
    )
    region = company_row["region"][0] if company_row["region"] else "Unknown"
    segment = company_row["segment"][0] if company_row["segment"] else "Unknown"
    total_credits = company_row["total_credits"]

    st.markdown(
        f":blue-badge[{account_type}] "
        f":violet-badge[{region}] "
        f":orange-badge[{segment}] "
        f":green-badge[{total_credits:,.0f} credits]"
    )

    # Summary metrics
    col1, col2, col3 = st.columns(3)
    with col1:
        st.metric("Total Credits", f"{total_credits:,.0f}")
    with col2:
        st.metric("Active Days", f"{company_row['active_days']:,}")
    with col3:
        growth = company_row["growth_score"]
        st.metric("Growth Score", f"{growth:+,.0f}")

    # Charts
    col1, col2 = st.columns(2)

    with col1:
        with st.container(border=True):
            st.markdown("**Daily usage**")
            st.line_chart(company_data, x="date", y="daily_credits", height=250)

    with col2:
        with st.container(border=True):
            st.markdown("**Cumulative usage**")
            chart_data = company_data.copy()
            chart_data["cumulative"] = chart_data["daily_credits"].cumsum()
            st.area_chart(chart_data, x="date", y="cumulative", height=250)


# =============================================================================
# Page Layout
# =============================================================================

# Load data
all_data = load_company_data()

st.markdown("# :material/business: Company Analytics")
st.caption("Track company adoption - usage, growth trends, and account details.")

# Filters
with st.container(border=True):
    st.markdown("**Filters**")

    # Company selection mode
    sort_mode = st.segmented_control(
        "Sort by",
        options=[
            "All companies",
            ":material/military_tech: Top spenders",
            ":material/trending_down: Top shrinkers",
            ":material/trending_up: Top gainers",
        ],
        default="All companies",
    )

    # Time window
    timeframe_options = {
        "All time": None,
        "Last 28 days": 28,
        "Last 7 days": 7,
    }
    timeframe = st.segmented_control(
        "Time window",
        options=list(timeframe_options.keys()),
        default="Last 28 days",
    )
    days_filter = timeframe_options.get(timeframe or "Last 28 days")

    # Account types
    account_types = st.pills(
        "Account types",
        options=ACCOUNT_TYPES,
        default=["Enterprise", "Growth", "Startup"],
        selection_mode="multi",
    )

# Determine sort order
if "Top shrinkers" in (sort_mode or ""):
    sort_by = "growth_asc"
elif "Top gainers" in (sort_mode or ""):
    sort_by = "growth_desc"
else:
    sort_by = "total_credits"

# Get filtered data
leaderboard = aggregate_companies(
    all_data,
    days=days_filter,
    account_types=account_types,
    sort_by=sort_by,
)

if leaderboard.empty:
    st.warning("No company data found for the selected filters.")
    st.stop()


# Convert columns to lists for MultiselectColumn display (shows nice colored chips)
for col in ["account_type", "region", "segment"]:
    leaderboard[col] = leaderboard[col].apply(_to_list)

# Companies dataframe
with st.container(border=True):
    # Guard against None: segmented_control returns None when deselected.
    timeframe_text = (
        timeframe.lower() if timeframe and timeframe != "All time" else "all time"
    )
    st.markdown(f"**Companies — {timeframe_text}**")

    # Selection dataframe with cell-click support
    selection = st.dataframe(
        leaderboard,
        column_config={
            "company_name": st.column_config.TextColumn(
                "Company (click to view details)",
                width="medium",
            ),
            "account_type": st.column_config.MultiselectColumn(
                "Type",
                options=ACCOUNT_TYPES,
                color="auto",
                width="small",
            ),
            "total_credits": st.column_config.NumberColumn(
                "Credits",
                format="%.0f",
            ),
            "growth_score": st.column_config.NumberColumn(
                "Growth",
                format="%+.0f",
                help="Credit change: second half vs first half of period",
            ),
            "usage_trend": st.column_config.LineChartColumn(
                "Trend",
                width="medium",
            ),
            "daily_avg": st.column_config.NumberColumn(
                "Daily Avg",
                format="%.1f",
            ),
            "active_days": st.column_config.NumberColumn(
                "Active Days",
                format="%d",
            ),
            "region": st.column_config.MultiselectColumn(
                "Region",
                options=REGIONS,
                color="auto",
            ),
            "segment": st.column_config.MultiselectColumn(
                "Segment",
                options=SEGMENTS,
                color="auto",
            ),
        },
        column_order=[
            "company_name",
            "account_type",
            "total_credits",
            "growth_score",
            "usage_trend",
            "daily_avg",
            "region",
            "segment",
        ],
        hide_index=True,
        on_select="rerun",
        selection_mode="single-cell",
        key="company_leaderboard",
    )

# Company drill-down via dialog when Company column cell is clicked
if selection.selection.cells:  # type: ignore[attr-defined]
    cell = selection.selection.cells[0]  # type: ignore[attr-defined]  # tuple: (row_index, column_name)
    row_idx, col_name = cell
    # Check if the clicked cell is in the company_name column
    if col_name == "company_name":
        selected_company = leaderboard.iloc[row_idx]["company_name"]
        company_row = leaderboard.iloc[row_idx]

        @st.dialog(f"{selected_company}", width="large")
        def show_company_dialog() -> None:
            render_company_dialog(
                selected_company,
                company_row=company_row,
                df=all_data,
            )

        show_company_dialog()



# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Runtime-related utility functions."""

from __future__ import annotations

from logging import getLogger
from typing import TYPE_CHECKING, Any, Final, cast

from streamlit import config
from streamlit.errors import MarkdownFormattedException, StreamlitAPIException

if TYPE_CHECKING:
    from streamlit.proto.ForwardMsg_pb2 import ForwardMsg

_LOGGER: Final = getLogger(__name__)


class MessageSizeError(MarkdownFormattedException):
    """Exception raised when a websocket message is larger than the configured limit."""

    def __init__(self, failed_msg_str: Any) -> None:
        msg = self._get_message(failed_msg_str)
        super().__init__(msg)

    def _get_message(self, failed_msg_str: Any) -> str:
        # This needs to have zero indentation otherwise the markdown will render incorrectly.
        return (
            f"""
**Data of size {len(failed_msg_str) / 1e6:.1f} MB exceeds the message size limit of
{get_max_message_size_bytes() / 1e6} MB.**

This is often caused by a large chart or dataframe. Please decrease the amount of data sent
to the browser, or increase the limit by setting the config option `server.maxMessageSize`.
[Click here to learn more about config options](https://docs.streamlit.io/develop/api-reference/configuration/config.toml).

_Note that increasing the limit may lead to long loading times and large memory consumption
of the client's browser and the Streamlit server._
"""
        ).strip("\n")


class BadDurationStringError(StreamlitAPIException):
    """Raised when a bad duration argument string is passed."""

    def __init__(self, duration: str) -> None:
        MarkdownFormattedException.__init__(
            self,
            "TTL string doesn't look right. It should be formatted as"
            f"`'1d2h34m'` or `2 days`, for example. Got: {duration}",
        )


def serialize_forward_msg(msg: ForwardMsg) -> bytes:
    """Serialize a ForwardMsg to send to a client.

    If the message is too large, it will be converted to an exception message
    instead.
    """

    msg_str = msg.SerializeToString()

    if len(msg_str) > get_max_message_size_bytes():
        # Overwrite the offending ForwardMsg.delta with an error to display.
        # This assumes that the size limit wasn't exceeded due to metadata.
        from streamlit.elements import exception

        msg_size_error = MessageSizeError(msg_str)
        _LOGGER.warning(
            "Websocket message size limit exceeded. Showing error to the user: %s",
            msg_size_error,
        )
        exception.marshall(msg.delta.new_element.exception, msg_size_error)
        # Deactivate caching for this error message:
        msg.metadata.cacheable = False
        msg_str = msg.SerializeToString()

    return msg_str


# This needs to be initialized lazily to avoid calling config.get_option() and
# thus initializing config options when this file is first imported.
_max_message_size_bytes: int | None = None


def get_max_message_size_bytes() -> int:
    """Returns the max websocket message size in bytes.

    This will lazyload the value from the config and store it in the global symbol table.
    """
    global _max_message_size_bytes  # noqa: PLW0603

    if _max_message_size_bytes is None:
        _max_message_size_bytes = config.get_option("server.maxMessageSize") * int(1e6)

    return cast("int", _max_message_size_bytes)



# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

from collections.abc import Callable, Mapping, Sequence
from pathlib import Path
from typing import TYPE_CHECKING, Literal, TypeAlias

from streamlit import config
from streamlit.errors import StreamlitAPIException
from streamlit.navigation.page import StreamlitPage
from streamlit.proto.ForwardMsg_pb2 import ForwardMsg
from streamlit.proto.Navigation_pb2 import Navigation as NavigationProto
from streamlit.runtime.metrics_util import gather_metrics
from streamlit.runtime.pages_manager import PagesManager
from streamlit.runtime.scriptrunner_utils.script_run_context import (
    ScriptRunContext,
    get_script_run_ctx,
)
from streamlit.string_util import is_emoji

if TYPE_CHECKING:
    from streamlit.proto.AppPage_pb2 import AppPage as AppPageProto
    from streamlit.source_util import PageHash, PageInfo

SectionHeader: TypeAlias = str
PageType: TypeAlias = str | Path | Callable[[], None] | StreamlitPage


def convert_to_streamlit_page(
    page_input: PageType,
) -> StreamlitPage:
    """Convert various input types to StreamlitPage objects."""
    if isinstance(page_input, StreamlitPage):
        return page_input

    if isinstance(page_input, str):
        return StreamlitPage(page_input)

    if isinstance(page_input, Path):
        return StreamlitPage(page_input)

    if callable(page_input):
        # Convert function to StreamlitPage
        return StreamlitPage(page_input)

    raise StreamlitAPIException(
        f"Invalid page type: {type(page_input)}. Must be either a string path, "
        "a pathlib.Path, a callable function, or a st.Page object."
    )


def pages_from_nav_sections(
    nav_sections: dict[SectionHeader, list[StreamlitPage]],
) -> list[StreamlitPage]:
    page_list = []
    for pages in nav_sections.values():
        page_list.extend(pages.copy())

    return page_list


def send_page_not_found(ctx: ScriptRunContext) -> None:
    msg = ForwardMsg()
    msg.page_not_found.page_name = ""
    ctx.enqueue(msg)


def _set_external_url(page_proto: AppPageProto, page: StreamlitPage) -> None:
    """Set external_url on the AppPage proto when the page targets an external URL."""
    external_url = page.external_url
    if external_url is not None:
        page_proto.external_url = external_url


@gather_metrics("navigation")
def navigation(
    pages: Sequence[PageType] | Mapping[SectionHeader, Sequence[PageType]],
    *,
    position: Literal["sidebar", "hidden", "top"] = "sidebar",
    expanded: bool | int = False,
) -> StreamlitPage:
    """
    Configure the available pages in a multipage app.

    Call ``st.navigation`` in your entrypoint file to define the available
    pages for your app. ``st.navigation`` returns the current page, which can
    be executed using ``.run()`` method.

    When using ``st.navigation``, your entrypoint file (the file passed to
    ``streamlit run``) acts like a router or frame of common elements around
    each of your pages. Streamlit executes the entrypoint file with every app
    rerun. To execute the current page, you must call the ``.run()`` method on
    the ``StreamlitPage`` object returned by ``st.navigation``.

    The set of available pages can be updated with each rerun for dynamic
    navigation. By default, ``st.navigation`` displays the available pages in
    the sidebar if there is more than one page. This behavior can be changed
    using the ``position`` keyword argument.

    As soon as any session of your app executes the ``st.navigation`` command,
    your app will ignore the ``pages/`` directory (across all sessions).

    Parameters
    ----------
    pages : Sequence[page-like], Mapping[str, Sequence[page-like]]
        The available pages for the app.

        To create a navigation menu with no sections or page groupings,
        ``pages`` must be a list of page-like objects. Page-like objects are
        anything that can be passed to ``st.Page`` or a ``StreamlitPage``
        object returned by ``st.Page``.

        To create labeled sections or page groupings within the navigation
        menu, ``pages`` must be a dictionary. Each key is the label of a
        section and each value is the list of page-like objects for
        that section. If you use ``position="top"``, each grouping will be a
        collapsible item in the navigation menu. For top navigation, if you use
        an empty string as a section header, the pages in that section will be
        displayed at the beginning of the menu before the collapsible sections.
        Section labels support GitHub-flavored Markdown as described in the
        ``title`` parameter of ``st.Page``.

        When you use a string or path as a page-like object, they are
        internally passed to ``st.Page`` and converted to ``StreamlitPage``
        objects. In this case, the page will have the default title, icon, and
        path inferred from its path or filename. To customize these attributes
        for your page, initialize your page with ``st.Page``.

    position : "sidebar", "top", or "hidden"
        The position of the navigation menu. If this is ``"sidebar"``
        (default), the navigation widget appears at the top of the sidebar. If
        this is ``"top"``, the navigation appears in the top header of the app.
        If this is ``"hidden"``, the navigation widget is not displayed.

        If there is only one page in ``pages``, the navigation will be hidden
        for any value of ``position``.

    expanded : bool or int
        Controls whether the navigation menu is expanded and how many items
        are visible when it's collapsed.

        This parameter is only used when ``position="sidebar"`` and the sidebar
        has other elements below the navigation menu. If the sidebar only
        contains the navigation menu, it will always be fully expanded. The
        following values are valid:

        - ``False`` (default): The navigation menu shows a maximum
          of ten pages when there are more than twelve pages. The menu is fully
          expanded when there are twelve or fewer pages. A collapsed menu has a
          "View X more" button at the bottom. An expanded menu that can be
          collapsed has a "View less" button at the bottom.

        - ``True``: The navigation menu will always be fully expanded, and no
          "View less" button will be displayed.

        - Positive integer: A positive integer specifies the maximum number of
          pages to display when the menu is collapsed. When there are at least
          three more pages than this integer, the menu will be collapsed.
          Otherwise, the menu will be fully expanded. ``expanded=10`` is
          equivalent to the default, ``expanded=False``.

        If the value of ``expanded`` changes between reruns, an expanded menu
        will stay expanded. If the change in value makes the menu newly
        collapsible, Streamlit will only add a "View less" button at the bottom.

    Returns
    -------
    StreamlitPage
        The current page selected by the user. To run the page, you must use
        the ``.run()`` method on it.

    Examples
    --------
    The following examples show different possible entrypoint files, each named
    ``streamlit_app.py``. An entrypoint file is passed to ``streamlit run``. It
    manages your app's navigation and serves as a router between pages.

    **Example 1: Use a callable or Python file as a page**

    You can declare pages from callables or file paths. If you pass callables
    or paths to ``st.navigation`` as a page-like objects, they are internally
    converted to ``StreamlitPage`` objects using ``st.Page``. In this case, the
    page titles, icons, and paths are inferred from the file or callable names.

    ``page_1.py`` (in the same directory as your entrypoint file):

    >>> import streamlit as st
    >>>
    >>> st.title("Page 1")

    ``streamlit_app.py``:

    >>> import streamlit as st
    >>>
    >>> def page_2():
    ...     st.title("Page 2")
    >>>
    >>> pg = st.navigation(["page_1.py", page_2])
    >>> pg.run()

    .. output::
        https://doc-navigation-example-1.streamlit.app/
        height: 200px

    **Example 2: Group pages into sections and customize them with ``st.Page``**

    You can use a dictionary to create sections within your navigation menu. In
    the following example, each page is similar to Page 1 in Example 1, and all
    pages are in the same directory. However, you can use Python files from
    anywhere in your repository. ``st.Page`` is used to give each page a custom
    title. For more information, see |st.Page|_.

    Directory structure:

    >>> your_repository/
    >>> ├── create_account.py
    >>> ├── learn.py
    >>> ├── manage_account.py
    >>> ├── streamlit_app.py
    >>> └── trial.py

    ``streamlit_app.py``:

    >>> import streamlit as st
    >>>
    >>> pages = {
    ...     "Your account": [
    ...         st.Page("create_account.py", title="Create your account"),
    ...         st.Page("manage_account.py", title="Manage your account"),
    ...     ],
    ...     "Resources": [
    ...         st.Page("learn.py", title="Learn about us"),
    ...         st.Page("trial.py", title="Try it out"),
    ...     ],
    ... }
    >>>
    >>> pg = st.navigation(pages)
    >>> pg.run()

    .. output::
        https://doc-navigation-example-2.streamlit.app/
        height: 300px


    **Example 3: Use top navigation**

    You can use the ``position`` parameter to place the navigation at the top
    of the app. This is useful for apps with a lot of pages because it allows
    you to create collapsible sections for each group of pages. The following
    example uses the same directory structure as Example 2 and shows how to
    create a top navigation menu.

    ``streamlit_app.py``:

    >>> import streamlit as st
    >>>
    >>> pages = {
    ...     "Your account": [
    ...         st.Page("create_account.py", title="Create your account"),
    ...         st.Page("manage_account.py", title="Manage your account"),
    ...     ],
    ...     "Resources": [
    ...         st.Page("learn.py", title="Learn about us"),
    ...         st.Page("trial.py", title="Try it out"),
    ...     ],
    ... }
    >>>
    >>> pg = st.navigation(pages, position="top")
    >>> pg.run()

    .. output::
        https://doc-navigation-top.streamlit.app/
        height: 300px

    **Example 4: Stateful widgets across multiple pages**

    Call widget functions in your entrypoint file when you want a widget to be
    stateful across pages. Assign keys to your common widgets and access their
    values through Session State within your pages.

    ``streamlit_app.py``:

    >>> import streamlit as st
    >>>
    >>> def page1():
    >>>     st.write(st.session_state.foo)
    >>>
    >>> def page2():
    >>>     st.write(st.session_state.bar)
    >>>
    >>> # Widgets shared by all the pages
    >>> st.sidebar.selectbox("Foo", ["A", "B", "C"], key="foo")
    >>> st.sidebar.checkbox("Bar", key="bar")
    >>>
    >>> pg = st.navigation([page1, page2])
    >>> pg.run()

    .. output::
        https://doc-navigation-multipage-widgets.streamlit.app/
        height: 350px

    .. |st.Page| replace:: ``st.Page``
    .. _st.Page: https://docs.streamlit.io/develop/api-reference/navigation/st.page

    """
    # Validate position parameter
    if not isinstance(position, str) or position not in {"sidebar", "hidden", "top"}:
        raise StreamlitAPIException(
            f'Invalid position "{position}". '
            'The position parameter must be one of "sidebar", "hidden", or "top".'
        )

    # Disable the use of the pages feature (ie disregard v1 behavior of Multipage Apps)
    PagesManager.uses_pages_directory = False

    return _navigation(pages, position=position, expanded=expanded)


def _navigation(
    pages: Sequence[PageType] | Mapping[SectionHeader, Sequence[PageType]],
    *,
    position: Literal["sidebar", "hidden", "top"],
    expanded: bool | int,
) -> StreamlitPage:
    if isinstance(pages, Sequence):
        converted_pages = [convert_to_streamlit_page(p) for p in pages]
        nav_sections = {"": converted_pages}
    else:
        nav_sections = {
            section: [convert_to_streamlit_page(p) for p in section_pages]
            for section, section_pages in pages.items()
        }
    page_list = pages_from_nav_sections(nav_sections)

    if not page_list:
        raise StreamlitAPIException(
            "`st.navigation` must be called with at least one `st.Page`."
        )

    default_page = None
    pagehash_to_pageinfo: dict[PageHash, PageInfo] = {}

    # Get the default page.
    for section_header in nav_sections:
        for page in nav_sections[section_header]:
            if page._default:
                if default_page is not None:
                    raise StreamlitAPIException(
                        "Multiple Pages specified with `default=True`. "
                        "At most one Page can be set to default."
                    )
                default_page = page

    if default_page is None:
        non_external_pages = [p for p in page_list if not p.is_external]
        if not non_external_pages:
            raise StreamlitAPIException(
                "At least one non-external page is required. "
                "External URL pages cannot be the default page."
            )
        default_page = non_external_pages[0]
        default_page._default = True

    ctx = get_script_run_ctx()
    if not ctx:
        # This should never run in Streamlit, but we want to make sure that
        # the function always returns a page
        default_page._can_be_called = True
        return default_page

    # Build the pagehash-to-pageinfo mapping.
    for section_header in nav_sections:
        for page in nav_sections[section_header]:
            script_path = str(page._page) if isinstance(page._page, Path) else ""

            script_hash = page._script_hash
            if script_hash in pagehash_to_pageinfo:
                # The page script hash is solely based on the url path
                # So duplicate page script hashes are due to duplicate url paths
                raise StreamlitAPIException(
                    f"Multiple Pages specified with URL pathname {page.url_path}. "
                    "URL pathnames must be unique. The url pathname may be "
                    "inferred from the filename, callable name, or title."
                )

            pagehash_to_pageinfo[script_hash] = {
                "page_script_hash": script_hash,
                "page_name": page.title,
                "icon": page.icon,
                "script_path": script_path,
                "url_pathname": page.url_path,
            }

    msg = ForwardMsg()
    # Handle position logic correctly
    if position == "hidden":
        msg.navigation.position = NavigationProto.Position.HIDDEN
    elif position == "top":
        msg.navigation.position = NavigationProto.Position.TOP
    elif position == "sidebar":
        # Only apply config override if position is sidebar
        if config.get_option("client.showSidebarNavigation") is False:
            msg.navigation.position = NavigationProto.Position.HIDDEN
        else:
            msg.navigation.position = NavigationProto.Position.SIDEBAR

    # Handle expanded parameter: must be bool or non-negative int
    if isinstance(expanded, bool):
        if expanded:
            msg.navigation.expanded = True
            # Don't set visible_items - leave it unset to use default
        else:
            # expanded is False - use default collapsed behavior
            msg.navigation.expanded = False
            # Don't set visible_items - leave it unset to use default
    elif isinstance(expanded, int):
        if expanded < 0:
            raise StreamlitAPIException(
                f"Invalid value for expanded: {expanded!r}. "
                "When using an int, expanded must be a non-negative integer."
            )
        if expanded == 0:
            # Documented default behavior: collapsed, default visible_items
            msg.navigation.expanded = False
            # Don't set visible_items - leave it unset to use default
        else:
            # Positive int: collapsed with a limited number of visible items
            msg.navigation.expanded = False
            msg.navigation.visible_items = expanded
    else:
        raise StreamlitAPIException(
            f"Invalid type for expanded: {type(expanded).__name__!s}. "
            "expanded must be a bool or a non-negative integer."
        )

    msg.navigation.sections[:] = nav_sections.keys()
    for section_header in nav_sections:
        for page in nav_sections[section_header]:
            p = msg.navigation.app_pages.add()
            p.page_script_hash = page._script_hash
            p.page_name = page.title
            p.icon = f"emoji:{page.icon}" if is_emoji(page.icon) else page.icon
            p.is_default = page._default
            p.section_header = section_header
            p.url_pathname = page.url_path
            p.is_hidden = page._visibility == "hidden"
            _set_external_url(p, page)

    # Inform our page manager about the set of pages we have
    ctx.pages_manager.set_pages(pagehash_to_pageinfo)
    found_page = ctx.pages_manager.get_page_script(
        fallback_page_hash=default_page._script_hash
    )

    page_to_return = None
    if found_page:
        found_page_script_hash = found_page["page_script_hash"]
        matching_pages = [
            p for p in page_list if p._script_hash == found_page_script_hash
        ]
        if len(matching_pages) > 0:
            page_to_return = matching_pages[0]

    # External pages cannot be accessed directly by URL
    if page_to_return and page_to_return.is_external:
        page_to_return = None

    if not page_to_return:
        send_page_not_found(ctx)
        page_to_return = default_page

    # Ordain the page that can be called
    page_to_return._can_be_called = True
    msg.navigation.page_script_hash = page_to_return._script_hash
    # Set the current page script hash to the page that is going to be executed
    ctx.set_mpa_v2_page(page_to_return._script_hash)

    # This will either navigation or yield if the page is not found
    ctx.enqueue(msg)

    return page_to_return



# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Shared path security utilities for preventing path traversal and SSRF attacks.

This module provides a centralized implementation for path validation that is
used by multiple parts of the codebase. Having a single implementation ensures
consistent security checks and avoids divergent behavior between components.

Security Context
----------------
These checks are designed to run BEFORE any filesystem operations (like
``os.path.realpath()``) to prevent Windows from triggering SMB connections
to attacker-controlled servers when resolving UNC paths. This prevents
SSRF attacks and NTLM hash disclosure.
"""

from __future__ import annotations

import os
import string


def is_unsafe_path_pattern(path: str) -> bool:
    r"""Return True if path contains UNC, absolute, drive, or traversal patterns.

    This function checks for dangerous path patterns that could lead to:
    - SSRF attacks via Windows UNC path resolution
    - NTLM hash disclosure via SMB connections
    - Path traversal outside intended directories
    - Path truncation via null bytes

    IMPORTANT: This check must run BEFORE any ``os.path.realpath()`` calls
    to prevent Windows from triggering SMB connections to attacker-controlled
    servers.

    Parameters
    ----------
    path : str
        The path string to validate.

    Returns
    -------
    bool
        True if the path contains unsafe patterns, False if it appears safe
        for further processing.

    Examples
    --------
    >>> is_unsafe_path_pattern("subdir/file.js")
    False
    >>> is_unsafe_path_pattern("\\\\server\\share")
    True
    >>> is_unsafe_path_pattern("../../../etc/passwd")
    True
    >>> is_unsafe_path_pattern("C:\\Windows\\system32")
    True
    """
    # Null bytes can be used for path truncation attacks
    if "\x00" in path:
        return True

    # UNC paths (Windows network shares, including \\?\ and \\.\ prefixes)
    if path.startswith(("\\\\", "//")):
        return True

    # Windows drive paths (e.g. C:\, D:foo) - on Windows, os.path.realpath() on a
    # drive path can trigger SMB connections if the drive is mapped to a network share.
    # This enables SSRF attacks and NTLM hash disclosure. We reject all drive-qualified
    # paths including drive-relative paths like "C:foo" which resolve against the current
    # directory of that drive. Checked on all platforms for defense-in-depth and
    # testability (CI runs on Linux).
    if len(path) >= 2 and path[0] in string.ascii_letters and path[1] == ":":
        return True

    # Rooted backslash or forward slash (absolute paths)
    if path.startswith(("\\", "/")):
        return True

    # Also check os.path.isabs for platform-specific absolute path detection
    if os.path.isabs(path):
        return True

    # Path traversal - check segments after normalizing separators
    normalized = path.replace("\\", "/")
    segments = [seg for seg in normalized.split("/") if seg]
    return ".." in segments



# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

from dataclasses import dataclass
from textwrap import dedent
from typing import TYPE_CHECKING, TypeAlias, cast

from streamlit.elements.lib.file_uploader_utils import enforce_filename_restriction
from streamlit.elements.lib.form_utils import current_form_id
from streamlit.elements.lib.layout_utils import create_layout_config
from streamlit.elements.lib.policies import (
    check_widget_policies,
    maybe_raise_label_warnings,
)
from streamlit.elements.lib.utils import (
    Key,
    LabelVisibility,
    compute_and_register_element_id,
    get_label_visibility_proto_value,
    to_key,
)
from streamlit.elements.widgets.file_uploader import _get_upload_files
from streamlit.proto.CameraInput_pb2 import CameraInput as CameraInputProto
from streamlit.proto.Common_pb2 import FileUploaderState as FileUploaderStateProto
from streamlit.proto.Common_pb2 import UploadedFileInfo as UploadedFileInfoProto
from streamlit.runtime.metrics_util import gather_metrics
from streamlit.runtime.scriptrunner import ScriptRunContext, get_script_run_ctx
from streamlit.runtime.state import (
    WidgetArgs,
    WidgetCallback,
    WidgetKwargs,
    register_widget,
)
from streamlit.runtime.uploaded_file_manager import DeletedFile, UploadedFile

if TYPE_CHECKING:
    from streamlit.delta_generator import DeltaGenerator
    from streamlit.elements.lib.layout_utils import WidthWithoutContent

SomeUploadedSnapshotFile: TypeAlias = UploadedFile | DeletedFile | None


@dataclass
class CameraInputSerde:
    def serialize(
        self,
        snapshot: SomeUploadedSnapshotFile,
    ) -> FileUploaderStateProto:
        state_proto = FileUploaderStateProto()

        if snapshot is None or isinstance(snapshot, DeletedFile):
            return state_proto

        file_info: UploadedFileInfoProto = state_proto.uploaded_file_info.add()
        file_info.file_id = snapshot.file_id
        file_info.name = snapshot.name
        file_info.size = snapshot.size
        file_info.file_urls.CopyFrom(snapshot._file_urls)

        return state_proto

    def deserialize(
        self, ui_value: FileUploaderStateProto | None
    ) -> SomeUploadedSnapshotFile:
        upload_files = _get_upload_files(ui_value)
        return_value = None if len(upload_files) == 0 else upload_files[0]
        if return_value is not None and not isinstance(return_value, DeletedFile):
            enforce_filename_restriction(return_value.name, [".jpg"])
        return return_value


class CameraInputMixin:
    @gather_metrics("camera_input")
    def camera_input(
        self,
        label: str,
        key: Key | None = None,
        help: str | None = None,
        on_change: WidgetCallback | None = None,
        args: WidgetArgs | None = None,
        kwargs: WidgetKwargs | None = None,
        *,  # keyword-only arguments:
        disabled: bool = False,
        label_visibility: LabelVisibility = "visible",
        width: WidthWithoutContent = "stretch",
    ) -> UploadedFile | None:
        r"""Display a widget that returns pictures from the user's webcam.

        Parameters
        ----------
        label : str
            A short label explaining to the user what this widget is used for.
            The label can optionally contain GitHub-flavored Markdown of the
            following types: Bold, Italics, Strikethroughs, Inline Code, Links,
            and Images. Images display like icons, with a max height equal to
            the font height.

            Unsupported Markdown elements are unwrapped so only their children
            (text contents) render. Common block-level Markdown (headings,
            lists, blockquotes) is automatically escaped and displays as
            literal text in labels.

            See the ``body`` parameter of |st.markdown|_ for additional,
            supported Markdown directives.

            For accessibility reasons, you should never set an empty label, but
            you can hide it with ``label_visibility`` if needed. In the future,
            we may disallow empty labels by raising an exception.

            .. |st.markdown| replace:: ``st.markdown``
            .. _st.markdown: https://docs.streamlit.io/develop/api-reference/text/st.markdown

        key : str, int, or None
            An optional string or integer to use as the unique key for
            the widget. If this is ``None`` (default), a key will be
            generated for the widget based on the values of the other
            parameters. No two widgets may have the same key. Assigning
            a key stabilizes the widget's identity and preserves its
            state across reruns even when other parameters change.

            A key lets you access the widget's value via
            ``st.session_state[key]`` (read-only). For more details, see
            `Widget behavior
            <https://docs.streamlit.io/develop/concepts/architecture/widget-behavior>`_.

            Additionally, if ``key`` is provided, it will be used as a
            CSS class name prefixed with ``st-key-``.

        help : str or None
            A tooltip that gets displayed next to the widget label. Streamlit
            only displays the tooltip when ``label_visibility="visible"``. If
            this is ``None`` (default), no tooltip is displayed.

            The tooltip can optionally contain GitHub-flavored Markdown,
            including the Markdown directives described in the ``body``
            parameter of ``st.markdown``.

        on_change : callable
            An optional callback invoked when this camera_input's value
            changes.

        args : list or tuple
            An optional list or tuple of args to pass to the callback.

        kwargs : dict
            An optional dict of kwargs to pass to the callback.

        disabled : bool
            An optional boolean that disables the camera input if set to
            ``True``. Default is ``False``.

        label_visibility : "visible", "hidden", or "collapsed"
            The visibility of the label. The default is ``"visible"``. If this
            is ``"hidden"``, Streamlit displays an empty spacer instead of the
            label, which can help keep the widget aligned with other widgets.
            If this is ``"collapsed"``, Streamlit displays no label or spacer.

        width : "stretch" or int
            The width of the camera input widget. This can be one of the
            following:

            - ``"stretch"`` (default): The width of the widget matches the
              width of the parent container.
            - An integer specifying the width in pixels: The widget has a
              fixed width. If the specified width is greater than the width of
              the parent container, the width of the widget matches the width
              of the parent container.

        Returns
        -------
        None or UploadedFile
            The UploadedFile class is a subclass of BytesIO, and therefore is
            "file-like". This means you can pass an instance of it anywhere a
            file is expected.

        Examples
        --------
        >>> import streamlit as st
        >>>
        >>> enable = st.checkbox("Enable camera")
        >>> picture = st.camera_input("Take a picture", disabled=not enable)
        >>>
        >>> if picture:
        ...     st.image(picture)

        .. output::
           https://doc-camera-input.streamlit.app/
           height: 600px

        """
        ctx = get_script_run_ctx()
        return self._camera_input(
            label=label,
            key=key,
            help=help,
            on_change=on_change,
            args=args,
            kwargs=kwargs,
            disabled=disabled,
            label_visibility=label_visibility,
            width=width,
            ctx=ctx,
        )

    def _camera_input(
        self,
        label: str,
        key: Key | None = None,
        help: str | None = None,
        on_change: WidgetCallback | None = None,
        args: WidgetArgs | None = None,
        kwargs: WidgetKwargs | None = None,
        *,  # keyword-only arguments:
        disabled: bool = False,
        label_visibility: LabelVisibility = "visible",
        width: WidthWithoutContent = "stretch",
        ctx: ScriptRunContext | None = None,
    ) -> UploadedFile | None:
        key = to_key(key)

        check_widget_policies(
            self.dg,
            key,
            on_change,
            default_value=None,
            writes_allowed=False,
        )
        maybe_raise_label_warnings(label, label_visibility)

        element_id = compute_and_register_element_id(
            "camera_input",
            user_key=key,
            key_as_main_identity=True,
            dg=self.dg,
            label=label,
            help=help,
            width=width,
        )

        camera_input_proto = CameraInputProto()
        camera_input_proto.id = element_id
        camera_input_proto.label = label
        camera_input_proto.form_id = current_form_id(self.dg)
        camera_input_proto.disabled = disabled
        camera_input_proto.label_visibility.value = get_label_visibility_proto_value(
            label_visibility
        )

        if help is not None:
            camera_input_proto.help = dedent(help)

        layout_config = create_layout_config(width=width)

        serde = CameraInputSerde()

        camera_input_state = register_widget(
            camera_input_proto.id,
            on_change_handler=on_change,
            args=args,
            kwargs=kwargs,
            deserializer=serde.deserialize,
            serializer=serde.serialize,
            ctx=ctx,
            value_type="file_uploader_state_value",
        )

        self.dg._enqueue(
            "camera_input", camera_input_proto, layout_config=layout_config
        )

        if isinstance(camera_input_state.value, DeletedFile):
            return None
        return camera_input_state.value

    @property
    def dg(self) -> DeltaGenerator:
        """Get our DeltaGenerator."""
        return cast("DeltaGenerator", self)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: streamlit/proto/Navigation.proto
# Protobuf Python Version: 7.34.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    7,
    34,
    1,
    '',
    'streamlit/proto/Navigation.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from streamlit.proto import AppPage_pb2 as streamlit_dot_proto_dot_AppPage__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n streamlit/proto/Navigation.proto\x1a\x1dstreamlit/proto/AppPage.proto\"\xeb\x01\n\nNavigation\x12\x10\n\x08sections\x18\x01 \x03(\t\x12\x1b\n\tapp_pages\x18\x02 \x03(\x0b\x32\x08.AppPage\x12&\n\x08position\x18\x03 \x01(\x0e\x32\x14.Navigation.Position\x12\x18\n\x10page_script_hash\x18\x04 \x01(\t\x12\x10\n\x08\x65xpanded\x18\x05 \x01(\x08\x12\x1a\n\rvisible_items\x18\x06 \x01(\rH\x00\x88\x01\x01\",\n\x08Position\x12\n\n\x06HIDDEN\x10\x00\x12\x0b\n\x07SIDEBAR\x10\x01\x12\x07\n\x03TOP\x10\x02\x42\x10\n\x0e_visible_itemsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'streamlit.proto.Navigation_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_NAVIGATION']._serialized_start=68
  _globals['_NAVIGATION']._serialized_end=303
  _globals['_NAVIGATION_POSITION']._serialized_start=241
  _globals['_NAVIGATION_POSITION']._serialized_end=285
# @@protoc_insertion_point(module_scope)
